#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import genx_fusion_analysis_base as gnx


# In[ ]:


from cartopy import feature as cfeature
import cartopy.crs as ccrs
import cartopy.io.shapereader as shpreader
import geopandas as gpd
import matplotlib as mpl
import matplotlib.pyplot as plt

import matplotlib.colors as mc
import colorsys

from matplotlib.lines import Line2D
from matplotlib.patches import Patch
import numpy as np
import pandas as pd

import copy
import functools
import yaml


# In[ ]:


prop_cycle = plt.rcParams['axes.prop_cycle']
colors = prop_cycle.by_key()['color']

mpl.rcParams['font.size'] = 14

η=0.4


# In[ ]:


nice_gray = "#333333"

mpl.rcParams['font.size'] = 12
mpl.rcParams['axes.titlesize'] = 12


mpl.rcParams['axes.edgecolor'] = nice_gray
mpl.rcParams['axes.titlecolor'] = nice_gray
mpl.rcParams['axes.labelcolor'] = nice_gray
mpl.rcParams['text.color'] = nice_gray
mpl.rcParams['xtick.color'] = nice_gray
mpl.rcParams['ytick.color'] = nice_gray
mpl.rcParams['xtick.labelcolor'] = nice_gray
mpl.rcParams['ytick.labelcolor'] = nice_gray
plt.rcParams['legend.title_fontsize'] = 11


# In[ ]:


gnx.categories_table['naturalgas_ccavgcf_moderate'] = gnx.ZCFCC
gnx.categories_table['naturalgas_ctavgcf_moderate'] = gnx.ZCFCT


# In[ ]:


gnx.color_map[gnx.ZCFCT] = 'mediumseagreen'


# In[ ]:


def int_list(a):
    return [int(el) for el in a]

def float_list(a):
    return [float(el) for el in a]


# In[ ]:


def fusion_net_time_avg_capacity_by_zone(scen, case):
    core_params_df = gnx.load_core_parameters_df(scen)
    
    net_avg_cap_factor = gnx.get_net_avg_cap(core_params_df, case)
    
    peak_thermal_cap = gnx.ts_capacity_by_zone(scen, case)
    return float_list(peak_thermal_cap * net_avg_cap_factor)


# In[ ]:


def get_category_sums(df, zone_names, variety="Capacity"):
    r"""
    df: Dataframe
        capacity dataframe
    zone_names: list of str
        required to identify generator types
    """
    drop_these = [gnx.FLEX]
    
    if variety == "Capacity" or variety == "Energy":  
        col_to_sum = "EndCap"
    elif variety == "EnergyCapacity":
        col_to_sum = "EndEnergyCap"

    category_sums = dict()

    for idx, row in df.iterrows():
        cat = gnx.generator_category(row.Resource, zone_names)
        val = float(row[col_to_sum])
        gnx.initialize_if_not_present(category_sums, cat, 0.0)
        category_sums[cat] += val

    gnx.drop_keys(category_sums, drop_these)        
    gnx.remove_nonpositive_entries(category_sums)
    gnx.initialize_if_not_present(category_sums, "Fusion", 0.0)
    gnx.initialize_if_not_present(category_sums, gnx.CCS, 0.0)

    
    return category_sums


# In[ ]:


@functools.cache
def capacity_sums_per_zone(scen, case):
    r"""
    """
    cap_df = gnx.get_capacity_df(scen, case)
    zone_names = gnx.get_zone_names()
    
    num_zones = len(zone_names)
    
    cat_sums = [get_category_sums(cap_df[cap_df.Zone == z], zone_names) for z in range(1, num_zones + 1)]
    
    fus_cap = fusion_net_time_avg_capacity_by_zone(scen, case)
    for z in range(num_zones):
        cat_sums[z]['Fusion'] = fus_cap[z]
        
    return cat_sums

def energy_sums(ps_df):
    keys = ps_df.keys()[1:-1]
    esums = ps_df.iloc[1].to_numpy()[1:-1]
    return pd.DataFrame(np.array([keys, esums]).T, columns=["Resource", "EndCap"])
    
def annual_energy_sum(scen, c):
    zone_names = gnx.get_zone_names()
    
    generated_energy_df = energy_sums(gnx.power_annual_sums(scen, c))
    recirculated_energy = gnx.get_total_recirc(scen, c)
    
    generated_energy = get_category_sums(generated_energy_df, zone_names)
    
    net_power = gnx.subtract_dicts(generated_energy, recirculated_energy)
    
    remove_storage_from_energy(net_power)
    return net_power

def remove_storage_from_energy(d):
    stors = ['Hydro-storage', gnx.BATTERY, gnx.METALAIR]
    gnx.drop_keys(d, stors)

@functools.cache
def annual_energy_sums_per_zone(scen, case):
    r"""
    """
    zone_names = gnx.get_zone_names()
    num_zones = len(zone_names)
    
    cap_df = gnx.get_capacity_df(scen, case)
    generated_energy_df = energy_sums(gnx.power_annual_sums(scen, case))  
    
    zonal_recirc = gnx.get_zonal_recirc(scen, case)
    zonal_recirc_as_dicts = [{'Fusion': float(x)} for x in zonal_recirc]
      
    raw_cat_sums = [get_category_sums(generated_energy_df[cap_df.Zone == z], zone_names) for z in range(1, num_zones + 1)]
    net_energy_sums = [gnx.subtract_dicts(raw_cat_sums[i], zonal_recirc_as_dicts[i]) for i in range(num_zones)]
    
    [remove_storage_from_energy(ns) for ns in net_energy_sums]
    return net_energy_sums


# In[ ]:


def load_final_network_capacity(scen, case):
    network = copy.deepcopy(gnx.load_original_network_connections(scen))
    nw_expansion = gnx.load_network_expansion(scen, case)
    network["Line_Max_Flow_MW"] += nw_expansion.New_Trans_Capacity
    return network
    
def number_of_transmission_lines(network_df):
    return len(network_df["Network_Lines"])

def line_max_power(network_df, line):
    return network_df.Line_Max_Flow_MW[line]

def line_max_possible_power(network_df, line):
    return network_df.Line_Max_Flow_MW[line] + network_df.Line_Max_Reinforcement_MW[line]

def line_final_power(original_network_df, final_network_df, line):
    orig = original_network_df.Line_Max_Flow_MW[line]
    new = final_network_df.New_Trans_Capacity[line]
    return orig + new
    
def line_between_points(ax, cs, start, end, network_df=None, **kwargs):
    lons, lats = np.array(cs).T
    ax.plot([lons[start], lons[end]], [lats[start], lats[end]], **kwargs)
    

       
def line_endpoints(network_df, line):
    num_zones = gnx.num_zones_from_network(network_df)
    
    z_zones = ["z"+str(i) for i in range(1, num_zones+1)]
    
    endpoints = network_df[z_zones].iloc[line]
    start = endpoints[endpoints == 1]
    end = endpoints[endpoints == -1]
    start_zone_ix = gnx.zz_zone_to_integer(start.keys()[0])
    end_zone_ix = gnx.zz_zone_to_integer(end.keys()[0])
    
    return (start_zone_ix-1, end_zone_ix-1)


def plot_network_and_expansion(ax, centroids, original_df, final_df, scale=2000):
    r"""
    ax: matplotlib axis
    centroids: list of (lon, lat)
    original_df, final_df: pd.DataFrame of network info
    scale: number, for width of lines drawn
    """
    n_lines = gnx.num_transmission_lines(original_df)
    for l in range(n_lines):
        endpoints = line_endpoints(original_df, l)
        
        lw = line_max_power(original_df, l) / scale
        lpw = line_max_possible_power(original_df, l) / scale
        lfw = line_final_power(original_df, final_df, l) / scale

        line_between_points(ax, centroids, *endpoints, lw=lpw, color="#C0C0C0", solid_capstyle='butt', zorder=-11)
        line_between_points(ax, centroids, *endpoints, lw=lfw, color="lightcoral", solid_capstyle='butt', zorder=-10)
        line_between_points(ax, centroids, *endpoints, lw=lw, color=nice_gray, solid_capstyle='butt', zorder=-9)
  


# In[ ]:


def plot_pie(ax, location, radius, data, colors):
    ax.pie(data, center=location, radius=radius, colors=colors, frame=True)


# In[ ]:


def plot_label(ax, location, **kwargs):
    ax.text(*location, label, **kwargs)


# In[ ]:


def sum_list_of_dicts(ld):
    """
    ld is [{'a':1}, {'a':2, 'b':5}]
    
    {'a':3, 'b':5}
    """
    results = dict()
    for d in ld:
        for k, v in d.items():
            if not k in results:
                results[k] = 0
            results[k] += v
    return results


# In[ ]:


scen = "MedOpp"

centroids_df = gnx.load_centroids()

model_regions = centroids_df.model_region.to_numpy()


# In[ ]:


num_zones = len(centroids_df)

network_df = copy.deepcopy(gnx.load_original_network_connections(scen))

n_xmission_lines = number_of_transmission_lines(network_df)

centroids = [gnx.centroid_for_zone(z, centroids_df) for z in range(1,num_zones+1)]

lons, lats = np.array(centroids).T


# In[ ]:


def plot_category_mix(ax, category_sums, zone, centroids_list, denominator):
    cs = category_sums[zone]
    centroid = centroids_list[zone]
    
    total_value = sum(cs.values())
    
    colors, values = np.array([(gnx.color_map[k], v) for k, v in cs.items()]).T
    plot_pie(ax, centroid, 7*np.sqrt(total_value/denominator), values, colors)


# In[ ]:


ipm_model_regions = gpd.read_file("Twenty-Zone_Model_regions.shp")
my_crs = ccrs.epsg(3310)


# ## Plots of zones with capacities

# In[ ]:


# pre-load keys and colormap for plot_capsums
scen = "HighOpp"
case = 10004

zonal_cap_sums = capacity_sums_per_zone(scen, case)
zonal_energy_sums = annual_energy_sums_per_zone(scen, case)


all_keys = set().union(*[set(l) for l in zonal_cap_sums + zonal_energy_sums])

filtered_colormap = {k: v for k, v in gnx.color_map.items() if k in all_keys}


# In[ ]:


def plot_capsums(ax, totdat, x0, y0=0, width=1, denom=1e6):
    subtotal = y0
    for k in list(filtered_colormap.keys())[::-1]:
        v = totdat.get(k, 0)
        v = v/denom
        ax.bar(x0, v, width, bottom=subtotal, color=gnx.color_map[k])
        subtotal += v
        
    ax.text(x0, y0-2, 'Total', ha='center', fontsize=8)


# In[ ]:


def scale_line(ax, x0, y0, scale, label):
    height = 1/scale
    half_height = height/2
    c = nice_gray
    ax.plot([x0, x0], [y0, y0 + height], color=c, solid_capstyle='butt', lw=0.5)
    ax.text(x0 - 0.1, y0 + half_height, label, rotation=90, color=c, ha='right', va='center', fontsize=8)
    


# In[ ]:


def preprend_hash(string):
    lines = string.split('\n')
    commented_lines = ['# ' + l for l in lines]
    return "\n".join(commented_lines)


# # Basic scenarios

# In[ ]:


import matplotlib.patches as patches


# In[ ]:


# https://stackoverflow.com/questions/40672088/matplotlib-customize-the-legend-to-show-squares-instead-of-rectangles
from matplotlib.legend_handler import HandlerPatch

# --- handlers ---

class HandlerRect(HandlerPatch):

    def create_artists(self, legend, orig_handle,
                       xdescent, ydescent, width, height,
                       fontsize, trans):

        x = width//2
        y = 0
        w = h = 10

        # create
        p = patches.Rectangle(xy=(x, y), width=w, height=h)

        # update with data from oryginal object
        self.update_prop(p, orig_handle, legend)

        # move xy to legend
        p.set_transform(trans)

        return [p]


# In[ ]:


def save_capacity_maps4(scen, case0, case1, product_name="Figure_10"):
       
    fig = plt.figure(figsize=(7, 5))
    ax0 = plt.subplot(2, 2, 1, projection=ccrs.PlateCarree(), frameon=False)
    ax1 = plt.subplot(2, 2, 3, projection=ccrs.PlateCarree(), frameon=False)
    ax2 = plt.subplot(2, 2, 2, projection=ccrs.PlateCarree(), frameon=False)
    ax3 = plt.subplot(2, 2, 4, projection=ccrs.PlateCarree(), frameon=False)


    ax0.set_title("Generation & transmission capacity", fontsize=8)
    ax2.set_title("Annual production", fontsize=8)
    
    def set_region(ax):
        ax.set_xlim(-102, -67)
        ax.set_ylim(25.5, 47.5)

    def add_map_background(ax):
        ax.add_geometries(ipm_model_regions.geometry, crs = my_crs, lw=0.25,
                  facecolor="#FAFAFA",  edgecolor='#E0E0E0', zorder = -20)

        ax.add_feature(cfeature.COASTLINE, lw=0.25, edgecolor="#E0E0E0", zorder=-21)
        
    def add_transmission_graph(ax, scen, case):
        original_network_df = gnx.load_original_network_connections(scen)
        final_network_df = gnx.load_network_expansion(scen, case)
        plot_network_and_expansion(ax, centroids, original_network_df, final_network_df, scale=4000)

    totalbar_config = {'x0': -69, 'y0': 28, 'width': 1.5}

    def plot_cap_and_trans_map(ax, scen, case):
        set_region(ax)
        
        zonal_cap_sums = capacity_sums_per_zone(scen, case)
        [plot_category_mix(ax, zonal_cap_sums, z, centroids, denominator=4000000) for z in range(num_zones)]
        totdat = sum_list_of_dicts(zonal_cap_sums)
        plot_capsums(ax=ax, totdat=totdat, denom=2e5, **totalbar_config)
        scale_line(ax, -70.5, totalbar_config['y0'], 0.1, label='2 TW')

        add_transmission_graph(ax, scen, case)
        add_map_background(ax)
        return zonal_cap_sums
    
    def plot_prod_map(ax, scen, case):
        set_region(ax)

        zonal_energy_sums = annual_energy_sums_per_zone(scen, case)
        [plot_category_mix(ax, zonal_energy_sums, z, centroids, denominator=8e9) for z in range(num_zones)]
        totdat = sum_list_of_dicts(zonal_energy_sums)
        y0 = 28
        plot_capsums(ax=ax, totdat=totdat, denom=4e8, **totalbar_config)
        scale_line(ax, -70.5, totalbar_config['y0'], 0.08, label='5 PWh')

        add_map_background(ax)
        return zonal_energy_sums

    zonal_cap_sums_0 = plot_cap_and_trans_map(ax0, scen, case0)
    zonal_cap_sums_1 = plot_cap_and_trans_map(ax1, scen, case1)
    
    zonal_energy_sums_0 = plot_prod_map(ax2, scen, case0)
    zonal_energy_sums_1 = plot_prod_map(ax3, scen, case1)
        
    
    # legends
    all_keys = set().union(*[set(l) for l in zonal_cap_sums_0 + zonal_cap_sums_1])
    filtered_colormap = {k: v for k, v in gnx.color_map.items() if k in all_keys}

    legend_elements = [Patch(facecolor=v, edgecolor=None,
                             label=k) for k, v in filtered_colormap.items()]
        
    legend1 = ax1.legend(handles=legend_elements,
                         loc=(0.5, -0.5),
                         ncol=4,
                         fontsize=8,
                         title="Generation Capacity")
    legend1.set_title('Generation capacity & annual production',prop={'size':8}) 
    
    transmission = {"Max. allowed": "#C0C0C0", "New": "lightcoral", "Existing": nice_gray}
    legend_elements = [Line2D([0.1,0.9],[0,0], color=v, lw=3,
                             label=k) for k, v in transmission.items()]
    
    legend2 = ax1.legend(handles=legend_elements, loc=(0, -0.5), ncol=1, fontsize=8, title='Transmission')
    legend2.set_title('Transmission', prop={'size':8})
    
    ax1.add_artist(legend1)
    ax1.add_artist(legend2)

    # plot labels
    labelx=-101
    labely=46
    for ax, letter in zip([ax0, ax1, ax2, ax3], ["a", "c", "b", "d"]):
        ax.text(labelx, labely, f"({letter})", fontsize=8)
    
    # saving
    scen0 = scen1 = scen
    
    labelx= -0.05
    labely=0.3
    ax0.text(labelx, labely, "100 GW fusion", transform=ax0.transAxes, rotation=90, fontsize=9)
    ax1.text(labelx, labely, "Without fusion", transform=ax1.transAxes, rotation=90, fontsize=9)
    
    
    plt.savefig(gnx.product_path(f"{product_name}.png"), bbox_inches='tight',
                dpi=450, facecolor='white', transparent=False)
    
    no_xmiss = f"""
Data on the transmission network and expansion are not listed here.

For those, see Step3/{scen}/template/Network.csv and
Step4/{scen}/case_{case0}/network_expansion.csv and
Step4/{scen}/case_{case1}/network_expansion.csv.

"""
    
    header = f"""{product_name}:
    
Data associated with {product_name}.pdf.
""" + gnx.generic_data_header + \
f"""
Generation capacity of each resource type in each zone.
Zones are listed in order.
Values for each resource type are in units of MW.
""" + no_xmiss


    for case, cap_sums in zip([case0, case1], [zonal_cap_sums_0, zonal_cap_sums_1]):
        output = yaml.dump(cap_sums)
        with open(gnx.product_path(f"{product_name}_capacity_case_{case}.yml"), 'w') as f:
            f.write(preprend_hash(header) + '\n')
            f.write(output)
        
    header = f"""{product_name}:
    
Data associated with {product_name}.pdf.
""" + gnx.generic_data_header + \
f"""
Net annual generation of each resource type in each zone.
Zones are listed in order.
Values for each resource type are in units of MWh.
""" + no_xmiss


    for case, en_sums in zip([case0, case1], [zonal_energy_sums_0, zonal_energy_sums_1]):
        output = yaml.dump(en_sums)
        with open(gnx.product_path(f"{product_name}_energy_case_{case}.yml"), 'w') as f:
            f.write(preprend_hash(header) + '\n')
            f.write(output)

    plt.show()


# In[ ]:


save_capacity_maps4("HighOpp", 10004, 0)


# # Supplemental information

# In[ ]:


def save_capacity_maps6(scen, case0, case1, case2, product_name="SUPP_XXX",
                        rowlabels=["100 GW fusion + storage", "100 GW fusion", "Without fusion"]):
       
    fig = plt.figure(figsize=(7, 7.5))
    ax0 = plt.subplot(3, 2, 1, projection=ccrs.PlateCarree(), frameon=False)
    ax1 = plt.subplot(3, 2, 3, projection=ccrs.PlateCarree(), frameon=False)
    ax2 = plt.subplot(3, 2, 2, projection=ccrs.PlateCarree(), frameon=False)
    ax3 = plt.subplot(3, 2, 4, projection=ccrs.PlateCarree(), frameon=False)
    ax4 = plt.subplot(3, 2, 5, projection=ccrs.PlateCarree(), frameon=False)
    ax5 = plt.subplot(3, 2, 6, projection=ccrs.PlateCarree(), frameon=False)  

    ax0.set_title("Generation & transmission capacity", fontsize=8)
    ax2.set_title("Annual production", fontsize=8)
    
    def set_region(ax):
        ax.set_xlim(-102, -67)
        ax.set_ylim(25.5, 47.5)

    def add_map_background(ax):
        ax.add_geometries(ipm_model_regions.geometry, crs = my_crs, lw=0.25,
                  facecolor="#FAFAFA",  edgecolor='#E0E0E0', zorder = -20)

        ax.add_feature(cfeature.COASTLINE, lw=0.25, edgecolor="#E0E0E0", zorder=-21)
        
    def add_transmission_graph(ax, scen, case):
        original_network_df = gnx.load_original_network_connections(scen)
        final_network_df = gnx.load_network_expansion(scen, case)
        plot_network_and_expansion(ax, centroids, original_network_df, final_network_df, scale=4000)

    totalbar_config = {'x0': -69, 'y0': 28, 'width': 1.5}

    def plot_cap_and_trans_map(ax, scen, case):
        set_region(ax)
        
        zonal_cap_sums = capacity_sums_per_zone(scen, case)
        [plot_category_mix(ax, zonal_cap_sums, z, centroids, denominator=4000000) for z in range(num_zones)]
        totdat = sum_list_of_dicts(zonal_cap_sums)
        plot_capsums(ax=ax, totdat=totdat, denom=2e5, **totalbar_config)
        scale_line(ax, -70.5, totalbar_config['y0'], 0.1, label='2 TW')

        add_transmission_graph(ax, scen, case)
        add_map_background(ax)
        return zonal_cap_sums
    
    def plot_prod_map(ax, scen, case):
        set_region(ax)

        zonal_energy_sums = annual_energy_sums_per_zone(scen, case)
        [plot_category_mix(ax, zonal_energy_sums, z, centroids, denominator=8e9) for z in range(num_zones)]
        totdat = sum_list_of_dicts(zonal_energy_sums)
        y0 = 28
        plot_capsums(ax=ax, totdat=totdat, denom=4e8, **totalbar_config)
        scale_line(ax, -70.5, totalbar_config['y0'], 0.08, label='5 PWh')

        add_map_background(ax)
        return zonal_energy_sums

    zonal_cap_sums_0 = plot_cap_and_trans_map(ax0, scen, case0)
    zonal_cap_sums_1 = plot_cap_and_trans_map(ax1, scen, case1)
    zonal_cap_sums_2 = plot_cap_and_trans_map(ax4, scen, case2)

    
    zonal_energy_sums_0 = plot_prod_map(ax2, scen, case0)
    zonal_energy_sums_1 = plot_prod_map(ax3, scen, case1)
    zonal_energy_sums_2 = plot_prod_map(ax5, scen, case2)
    
    
    # legends
    all_keys = set().union(*[set(l) for l in zonal_cap_sums_0 + zonal_cap_sums_1])
    filtered_colormap = {k: v for k, v in gnx.color_map.items() if k in all_keys}

    legend_elements = [Patch(facecolor=v, edgecolor=None,
                             label=k) for k, v in filtered_colormap.items()]
        
    legend1 = ax4.legend(handles=legend_elements,
                         loc=(0.5, -0.5),
                         ncol=4,
                         fontsize=8,
                         title="Generation Capacity")
    legend1.set_title('Generation capacity & annual production',prop={'size':8}) 
    
    transmission = {"Max. allowed": "#C0C0C0", "New": "lightcoral", "Existing": nice_gray}
    legend_elements = [Line2D([0.1,0.9],[0,0], color=v, lw=3,
                             label=k) for k, v in transmission.items()]
    
    legend2 = ax4.legend(handles=legend_elements, loc=(0, -0.5), ncol=1, fontsize=8, title='Transmission')
    legend2.set_title('Transmission', prop={'size':8})
    
    ax4.add_artist(legend1)
    ax4.add_artist(legend2)

    # plot labels
    labelx=-101
    labely=46
    for ax, letter in zip([ax0, ax1, ax2, ax3, ax4, ax5], ["a", "c", "b", "d", "e", "f"]):
        ax.text(labelx, labely, f"({letter})", fontsize=8)
    
    # saving
    scen0 = scen1 = scen
    
    labelx= -0.05
    labely=0.3
    ax0.text(labelx, labely-0.3, rowlabels[0], transform=ax0.transAxes, rotation=90, fontsize=9)
    ax1.text(labelx, labely, rowlabels[1], transform=ax1.transAxes, rotation=90, fontsize=9)
    ax4.text(labelx, labely, rowlabels[2], transform=ax4.transAxes, rotation=90, fontsize=9)
    
    
    plt.savefig(gnx.product_path(f"{product_name}.png"), bbox_inches='tight',
                dpi=450, facecolor='white', transparent=False)
    
    no_xmiss = f"""
Data on the transmission network and expansion are not listed here.

For those, see Step3/{scen}/template/Network.csv and
Step4/{scen}/case_{case0}/network_expansion.csv and
Step4/{scen}/case_{case1}/network_expansion.csv and
Step4/{scen}/case_{case2}/network_expansion.csv.

"""
    
    header = f"""{product_name}:
    
Data associated with {product_name}.pdf.
""" + gnx.generic_data_header + \
f"""
Generation capacity of each resource type in each zone.
Zones are listed in order.
Values for each resource type are in units of MW.
""" + no_xmiss


    for case, cap_sums in zip([case0, case1, case2], [zonal_cap_sums_0, zonal_cap_sums_1, zonal_cap_sums_2]):
        output = yaml.dump(cap_sums)
        with open(gnx.product_path(f"{product_name}_capacity_case_{case}.yml"), 'w') as f:
            f.write(preprend_hash(header) + '\n')
            f.write(output)
        
    header = f"""{product_name}:
    
Data associated with {product_name}.pdf.
""" + gnx.generic_data_header + \
f"""
Net annual generation of each resource type in each zone.
Zones are listed in order.
Values for each resource type are in units of MWh.
""" + no_xmiss


    for case, en_sums in zip([case0, case1, case2], [zonal_energy_sums_0, zonal_energy_sums_1, zonal_energy_sums_2]):
        output = yaml.dump(en_sums)
        with open(gnx.product_path(f"{product_name}_energy_case_{case}.yml"), 'w') as f:
            f.write(preprend_hash(header) + '\n')
            f.write(output)

    plt.show()


# In[ ]:


save_capacity_maps6("MedOpp", 10024, 10004, 0, product_name="SUPP_6map_MedOpp")


# In[ ]:


save_capacity_maps6("LowOpp", 10024, 10004, 0, product_name="SUPP_6map_LowOpp")


# In[ ]:


save_capacity_maps6("HighOpp", 10024, 10004, 0, product_name="SUPP_6map_HighOpp")


# In[ ]:


save_capacity_maps6("MedOpp", 30004, 10004, 40004, product_name="SUPP_6map_MedOpp_Transvar", rowlabels=["No new transmission", "Standard", "Unlimited transmission"])


# In[ ]:




