#!/usr/bin/env python
# coding: utf-8

import genx_fusion_analysis_base as gnx

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.lines import Line2D
from matplotlib.patches import Rectangle
import matplotlib as mpl

import pandas as pd

colors = gnx.colors

nice_gray = "#333333"

#mpl.rcParams['font.family'] = 'sans-serif'
print_plot_text = 8
mpl.rcParams['font.size'] = print_plot_text
mpl.rcParams['axes.titlesize'] = print_plot_text


mpl.rcParams['axes.edgecolor'] = nice_gray
mpl.rcParams['axes.titlecolor'] = nice_gray
mpl.rcParams['axes.labelcolor'] = nice_gray
mpl.rcParams['text.color'] = nice_gray
mpl.rcParams['xtick.color'] = nice_gray
mpl.rcParams['ytick.color'] = nice_gray
mpl.rcParams['xtick.labelsize'] = print_plot_text
mpl.rcParams['ytick.labelsize'] = print_plot_text
mpl.rcParams['xtick.labelcolor'] = nice_gray
mpl.rcParams['ytick.labelcolor'] = nice_gray
mpl.rcParams['legend.title_fontsize'] = print_plot_text


def mean_capacity_factor(scen, resource):
    gen_var = gnx.load_generators_variability(scen)
    return np.mean(gen_var[resource])


x_all = np.r_[10, 20, 50, 100, 150, 200, 250, 300, 350]
x_std = x_all[:-1]


a_std = np.arange(1, 9)
a_all = np.arange(1, 10)


cases_nostor_pes = 0 + a_all
cases_nostor_opt = 100 + a_all
cases_nostor_mid = 10000 + a_all


def max_plant_cost(scen, x, cases):
    duals = gnx.grab_duals(scen, cases)
    gen = gnx.grab_data("generation", scen, cases) / 1e3
    gen_per_cap = gen / x
    return x, (duals + gnx.base_gen_cost * gen_per_cap)/1e3/gnx.CAPEX_TO_ANN


fig, ax = plt.subplots(1, 3, figsize=(6, 2.5), sharex=True, sharey=False)

styles = ['solid', 'dashed', 'dotted']
colors = gnx.colors

for i, scen in enumerate(("LowOpp", "MedOpp", "HighOpp")):
    ax[i].plot(*max_plant_cost(scen, x_all, cases_nostor_opt),
               lw=1, ls='solid', label='Optimistic')
    ax[i].plot(*max_plant_cost(scen, x_all, cases_nostor_mid),
               lw=1, ls='dashed', label='Mid-range')
    ax[i].plot(*max_plant_cost(scen, x_all, cases_nostor_pes),
               lw=1, ls='dotted', label='Pessimistic')

xp, yp = max_plant_cost("MedOpp", x_all, cases_nostor_mid)
ax[1].scatter(xp[[0, 3]], yp[[0, 3]], 10, color=colors[1])

legend_x = 1.25


def thinspines(ax):
    for k, v in ax.spines.items():
        v.set_linewidth(0.5)


for axx in ax:
    axx.set_xlim(0, 350)
    maxy = 8500
    axx.set_ylim(0, maxy)
    axx.set_xticks([0, 100, 200, 300])
    axx.set_xticks([10, 20, 50, 150, 250], minor=True)
    axx.set_yticks([1000, 3000, 5000, 7000], minor=True)
    axx.spines["top"].set_visible(False)
    thinspines(axx)
    axx.tick_params(direction='out', length=2, width=0.5)


fig.supxlabel("Fusion capacity / GW")
ax[0].set_ylabel("Capital cost threshold\nfor marginal plant / (USD / kW)")

for axx in ax:
    secax = axx.secondary_yaxis('right', functions=(
        gnx.capex_to_ann, gnx.ann_to_capex), color='gray')
    secax.tick_params(direction='out', length=2, width=0.5)
    yticks = range(0, 700, 100)
    secax.set_yticks(yticks)
    secax.set_yticklabels(yticks, fontsize=7)
    thinspines(secax)

secax.set_ylabel(
    'Annualized cost threshold\nfor marginal plant / (USD / kW·yr)', color='gray')

for axx, label, title in zip(ax, ("a", "b", "c"), ("Low market opportunity", "Medium market opp.", "High market opp.")):
    axx.text(0.075, 0.95, f"({label})", transform=axx.transAxes)
    axx.set_title(title)


ax[0].text(100, 4400, "Optimistic", rotation=-14)
ax[0].text(100, 2900, "Mid-range", rotation=-14)
ax[0].text(100, 1300, "Pessimistic", rotation=-15)


plt.tight_layout(pad=0.5)
chart_name = "Figure_2"
plt.savefig(gnx.product_path(f'{chart_name}.pdf'), bbox_inches="tight")
plt.show()


data = {'Capacity': x_all}


for scen in ("LowOpp", "MedOpp", "HighOpp"):
    data[f"{scen} Opt"] = max_plant_cost(scen, x_all, cases_nostor_opt)[1]
    data[f"{scen} Mid"] = max_plant_cost(scen, x_all, cases_nostor_mid)[1]
    data[f"{scen} Pes"] = max_plant_cost(scen, x_all, cases_nostor_pes)[1]



df = pd.DataFrame(data)

data2 = df.to_numpy()

columns = ", ".join(df.keys())

header = """Figure 2:
Marginal value of the three reference fusion plants in the three main scenarios,
as a function of the system fusion capacity.
""" + gnx.generic_data_header + \
    f"""

Capacity has units of GW
Other columns [scenario, reference reactor] have units of 2019$/kW.

Note that to obtain the value in terms of 2019$/(kW·yr) (the right axis), multiply by {gnx.CAPEX_TO_ANN}.

""" + \
    columns

np.savetxt(gnx.product_path(f"{chart_name}.csv"),
           data2, fmt="%.1f", header=header, delimiter=',')

