#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import genx_fusion_analysis_base as gnx


# In[ ]:


import calendar

import numpy as np
import matplotlib as mpl
import matplotlib.colors as mcolors

import matplotlib.pyplot as plt
from matplotlib.lines import Line2D
import pandas as pd

from mpl_toolkits.axes_grid1 import make_axes_locatable


# In[ ]:


def num_to_lowercase(n):
    return chr(ord("`") + 1 + n)


# In[ ]:


prop_cycle = plt.rcParams['axes.prop_cycle']
colors = prop_cycle.by_key()['color']

#mpl.rcParams['font.size'] = 14


nice_gray = "#333333"

left_label_size=11

mpl.rcParams['font.size'] = 14
mpl.rcParams['axes.titlesize'] = 14


mpl.rcParams['axes.edgecolor'] = nice_gray
mpl.rcParams['axes.titlecolor'] = nice_gray
mpl.rcParams['axes.labelcolor'] = nice_gray
mpl.rcParams['text.color'] = nice_gray
mpl.rcParams['xtick.color'] = nice_gray
mpl.rcParams['ytick.color'] = nice_gray
mpl.rcParams['xtick.labelsize'] = 14
mpl.rcParams['ytick.labelsize'] = 14
mpl.rcParams['xtick.labelcolor'] = nice_gray
mpl.rcParams['ytick.labelcolor'] = nice_gray
mpl.rcParams['legend.title_fontsize'] = 13


# In[ ]:





# In[ ]:


UTC_TO_EDT = -6


# In[ ]:


η = 0.4


# In[ ]:


med_scen_parameters_df = gnx.load_core_parameters_df("MedOpp")


# In[ ]:


def truncate_colormap(cmap, minval=0.0, maxval=1.0, n=-1):
    if n == -1:
        n = cmap.N
    new_cmap = mcolors.LinearSegmentedColormap.from_list(
         'trunc({name},{a:.2f},{b:.2f})'.format(name=cmap.name, a=minval, b=maxval),
         cmap(np.linspace(minval, maxval, n)))
    return new_cmap


# In[ ]:


def set_monthlabels(axs):
    
    ax_last = axs[-1]
    
    month_ends = np.array([0,  31,  59,  90, 120, 151, 181, 212, 243, 273, 304, 334, 365])
    centers = month_ends[:-1] + (month_ends[1:] - month_ends[:-1]) / 2
    
    for ax in axs:
        ax.set_xticks(month_ends, minor=True)
        ax.tick_params(length=3, which='minor')
        ax.set_xticks(centers, minor=False)
        ax.tick_params(length=0, which='major')
        
    ax_last.set_xticklabels((calendar.month_abbr[i] for i in range(1, 13)))
    
    #ax.set_xlabel('Day of year')


# In[ ]:


def remove_zone_name(generator):
    zone_name = gnx.detect_zone_name(generator)
    zone_prefix = zone_name + "_"
    return generator[len(zone_prefix):]

def detect_substring(main_string, substrings):
    r"""First of the substrings found in the main string
    
    Note: right now doesn't assess the position of the substring,
    i.e. whether it's a prefix.
    """
    for s in substrings:
        if s in main_string or s.lower() in main_string:
            return s

def detect_zone_name(generator, zone_names):
    r"""Returns the first zone name string found in the generator
    FRCC_conventional_hydroelectric_1 --> FRCC
    """
    return detect_substring(generator, zone_names)      
    
def remove_number_suffixes(a_string):
    r"""
    offshorewind_class1_moderate_fixed_0_180_1 ->
        offshorewind_class1_moderate_fixed
    """
    segs = a_string.split("_")
    for i, s in enumerate(segs[::-1]):
        if i == 0 and not s.isnumeric():
            return "_".join(segs)
        if i > 0 and not s.isnumeric():
            return "_".join(segs[:-i])


# In[ ]:


def fusion_resource_name(z):
    return (gnx.ZONE_NAMES[z]+'_fusion').lower()

def fusion_generation_in_zone(power_df, zone):
    resource_name = fusion_resource_name(zone)
    generated = power_df[resource_name]
    return generated.to_numpy()

def fusion_recirculation_in_zone(recirc_df, zone):
    resource_name = fusion_resource_name(zone)
    recirc = recirc_df[resource_name]
    return recirc.to_numpy()

def net_fusion_generation_in_zone(power_df, recirc_df, zone):
    gen = fusion_generation_in_zone(power_df, zone)
    rec = fusion_recirculation_in_zone(recirc_df, zone)
    return gen - rec

def fusion_core_power_in_zone(corepower_df, zone):
    resource_name = fusion_resource_name(zone)
    generated = corepower_df[resource_name]
    return generated.to_numpy()

def fusion_generation_capacity_in_zone(df, zone):
    resource_name = fusion_resource_name(zone)
    cap = df[df.Resource == resource_name].EndCap
    return cap.to_numpy()[0]


# In[ ]:


def storage_soc_series_for_zone(scen, case, zone):
    resource_name = fusion_resource_name(zone)
    df = gnx.ts_soc_df(scen, case)
    duration = gnx.ts_duration_for_zone(scen, case, zone)
    
    raw_series = df[resource_name]
    normed_series = raw_series / np.max(raw_series)
    duration_series = normed_series * duration
    return duration_series.to_numpy()


# In[ ]:


def get_interesting_zones(cap_by_zone):
    r"""Get a few top zones for the run"""
    max_cap = max(cap_by_zone)
    normalized_cap = cap_by_zone / max_cap
    return np.arange(len(cap_by_zone))[normalized_cap > 0.25]


# In[ ]:


def get_top_zones(cap_by_zone):
    return np.argmax(cap_by_zone)


# In[ ]:


def shift_data(x):
    y = np.roll(x, UTC_TO_EDT)
    return y


# In[ ]:


def shift_and_reshape_data(x):
    y = shift_data(x)
    z = y.reshape(365, 24).T
    return z


# In[ ]:


positive_cmap = 'plasma_r'
diverging_cmap = mpl.cm.BrBG_r
hours_ticklist = [0,8,16,24]


# In[ ]:


def data_for_reactor_in_zone(scen, case, z):
    
    param_df =  gnx.load_core_parameters_df(scen)
    capacity_df = gnx.get_capacity_df(scen, case)
    corepower_df = gnx.fusion_corepower_hourly(scen, case)
    power_df = gnx.power_hourly(scen, case)
    recirc_df = gnx.fusion_recirc(scen, case)
    
    ts_capacity = gnx.ts_capacity_by_zone(scen, case)[z]
    gross_gen_cap = fusion_generation_capacity_in_zone(capacity_df, z)
    expected_gen_cap = gnx.get_long_run_gen_to_core_ratio(param_df, case)
    net_avg_cap = gnx.get_net_avg_cap(param_df, case)
    long_run_average_power = ts_capacity * net_avg_cap
    min_net_power_fraction = gnx.get_minimum_net_power_fraction(param_df, case)
    minimum_plausible_normalized_value = η * min_net_power_fraction / net_avg_cap
    
    passive_recirc_fraction = gnx.get_passive_recirc_fraction(param_df, case)
    passive_recirc_power = -ts_capacity * η * passive_recirc_fraction
    max_plausible_power = passive_recirc_power + gross_gen_cap
    max_plausible_power_ratio = max_plausible_power / long_run_average_power    
    
    raw_fusion_core_power = fusion_core_power_in_zone(corepower_df, z)
    normalized_fusion_core_power = raw_fusion_core_power / ts_capacity
    
    raw_fusion_generation = fusion_generation_in_zone(power_df, z)
    norm_fusion_generation = raw_fusion_generation / gross_gen_cap
    
    raw_fusion_recirc = fusion_recirculation_in_zone(recirc_df, z)
    #norm_recirc = raw_fusion_recirc / (-ts_capacity * minimum_plausible_normalized_value)
    norm_recirc = raw_fusion_recirc / np.max(raw_fusion_recirc)
      
    raw_net_fusion_generation = net_fusion_generation_in_zone(power_df, recirc_df, z)
    norm_net_fusion_generation = raw_net_fusion_generation / long_run_average_power
    
    storage_duration_soc = storage_soc_series_for_zone(scen, case, z)
    
    data = {'Norm. fusion core power': normalized_fusion_core_power,
            #'Recirculating power': 
            'Norm. gross power generation': norm_fusion_generation, 
            'Norm. recirc power': norm_recirc, 
            'Generator OP ratio': gross_gen_cap / (ts_capacity * expected_gen_cap),
            
            'Raw net fusion generation': raw_net_fusion_generation,
            'Norm. net fusion generation': norm_net_fusion_generation,
            'Min. plaus. norm. net gen': minimum_plausible_normalized_value,
            'Max. plaus. norm. net gen': max_plausible_power_ratio,
            'Storage duration SOC': storage_duration_soc,
           }
    
    return data


def add_colorbar_on_right(fig, ax, plot, ticks, ticklabels=None, minors=None):
    divider = make_axes_locatable(ax)
    ax_cb = divider.new_horizontal(size="1.5%", pad=0.05)
    fig.add_axes(ax_cb)
    my_cbar = plt.colorbar(plot, cax=ax_cb, ticks=ticks)
    if ticklabels is not None:
        my_cbar.ax.set_yticklabels(ticklabels, fontsize=10)
    
    if minors is not None:
        my_cbar.ax.set_yticks(minors, minor=True)
        my_cbar.ax.set_yticklabels(["",""], minor=True)
    return ax_cb
        
    
common_opts = {'shading': 'flat', 'edgecolors': 'none', 'rasterized': True, 'snap': True}

def plot_electrical_load(fig, ax, scen, zone):
    raw_data = gnx.load_data_for_zone(zone + 1, scen)
    data = 1.0 * shift_and_reshape_data(raw_data) 
    data /= np.max(data)
    series = data
    plot = ax.pcolormesh(series, cmap='Greens', vmin=0, vmax=1, **common_opts)
    ax.set_ylabel("Load", fontsize=left_label_size)
    ax.set_yticks(hours_ticklist)
    add_colorbar_on_right(fig, ax, plot, [0, 1])
    return shift_data(raw_data) / np.max(raw_data)

def plot_price(fig, ax, scen, kase, zone, maxprice=500):
    raw_data = gnx.load_price_data_df(scen, kase)[str(zone+1)]
    data = 1.0 * shift_and_reshape_data(raw_data) 
    #data /= np.max(data)
    series = data
    plot = ax.pcolormesh(series, cmap='viridis', vmin=0, vmax=maxprice, **common_opts)
    ax.set_ylabel("Price", fontsize=left_label_size)
    ax.set_yticks(hours_ticklist)
    my_cbar = add_colorbar_on_right(fig, ax, plot, [0, maxprice])
    ax.my_cbar = my_cbar
    return shift_data(raw_data)

def plot_core_power_chart(fig, ax, plant_data):
    raw_series = plant_data["Norm. fusion core power"]
    series = shift_and_reshape_data(raw_series) 
    plot = ax.pcolormesh(series, cmap=positive_cmap, vmin=0, vmax=1, **common_opts)
    ax.set_yticks(hours_ticklist)
    add_colorbar_on_right(fig, ax, plot, ticks=[0, 1])
    return shift_data(raw_series)
    
def plot_gross_generation_chart(fig, ax, plant_data):
    raw_series = plant_data["Norm. gross power generation"]
    series = shift_and_reshape_data(raw_series)
    plot = ax.pcolormesh(series, cmap='Oranges', vmin=0, vmax=1, **common_opts)
    ax.set_yticks(hours_ticklist)
    add_colorbar_on_right(fig, ax, plot, ticks=[0, 1])
    return shift_data(raw_series)
    
def plot_storage_chart(fig, ax, plant_data):
    raw_series = plant_data['Storage duration SOC']
    storage_max = np.max(raw_series)
    series = shift_and_reshape_data(raw_series)
    plot = ax.pcolormesh(series, cmap='Greys', vmin=0, vmax=storage_max,  **common_opts)
    ax.set_yticks(hours_ticklist)
    my_cbar = add_colorbar_on_right(fig, ax, plot, ticks=[0,5])
    ax.my_cbar = my_cbar
    return shift_data(raw_series)
    
def plot_recirc_chart(fig, ax, plant_data):
    raw_series = plant_data["Norm. recirc power"]
    series = shift_and_reshape_data(raw_series)
    plot = ax.pcolormesh(series, cmap='Blues', vmin=0, vmax=1,  **common_opts)
    ax.set_yticks(hours_ticklist)
    add_colorbar_on_right(fig, ax, plot, ticks=[0, 1])
    return shift_data(raw_series)

def truncate_colormap(cmap, minval=0.0, maxval=1.0, n=-1):
    if n == -1:
        n = cmap.N
    new_cmap = mcolors.LinearSegmentedColormap.from_list(
         'trunc({name},{a:.2f},{b:.2f})'.format(name=cmap.name, a=minval, b=maxval),
         cmap(np.linspace(minval, maxval, n)))
    return new_cmap
    
    
def plot_net_generation_chart(fig, ax, plant_data):
    raw_series = plant_data["Norm. net fusion generation"]
    series = shift_and_reshape_data(raw_series)
    
    new_cmap = truncate_colormap(diverging_cmap, 0.25+0.125, 1)
    
    plot = ax.pcolormesh(series, cmap=new_cmap, vmin=-0.5, vmax=2, **common_opts)
    ax.set_yticks(hours_ticklist)
#     max_plaus = plant_data["Max. plaus. norm. net gen"]
#     min_plaus = plant_data["Min. plaus. norm. net gen"]
    my_ticks = [-0.5,0,1,2]
    my_minor_ticks = [max(raw_series), min(raw_series)]
    my_cbar = add_colorbar_on_right(fig, ax, plot,
                                    ticks=my_ticks,
                                    ticklabels=['-½', '0', '1', '2'],
                                    minors=my_minor_ticks)
    ax.my_cbar = my_cbar
    return shift_data(raw_series)


# In[ ]:


cases = [4]

scen = "MedOpp"
param_df =  gnx.load_core_parameters_df(scen)

top_zones = {case: get_top_zones(gnx.ts_capacity_by_zone(scen, case)) for case in cases}


# In[ ]:


plant_data = [data_for_reactor_in_zone(scen, case, top_zones[case]) for case in cases]


# In[ ]:


cases = [324, 10024, 224] # with storage

caselabels = ["Pessimistic", "Mid-range", "Optimistic"]
scen = "MedOpp"

top_zones = {case: get_top_zones(gnx.ts_capacity_by_zone(scen, case)) for case in cases}
plant_data = [data_for_reactor_in_zone(scen, case, top_zones[case]) for case in cases]

fig, ax = plt.subplots(2*len(cases)+1, 1, figsize=(16,10), sharex=True)
fig.tight_layout(h_pad=0.2)

dump = dict()
dump["Normalized load"] = plot_electrical_load(fig, ax[0], scen, top_zones[cases[0]])

for i, plant_d in enumerate(plant_data):
    j = 2*i + 1
    key = caselabels[i]
    dump[key + " core: core power"] = plot_core_power_chart(fig, ax[j], plant_d)
    dump[key + " core: net generation"] = plot_net_generation_chart(fig, ax[j+1], plant_d) 
    
# for i, lab in enumerate(caselabels):    
#     ax[2*i+1].set_ylabel(lab, horizontalalignment='right')

[x.set_ylabel("Core pow.") for x in ax[1::2]]
[x.set_ylabel("Net gen.") for x in ax[2::2]]

line_x = -0.047
fig.add_artist(Line2D([line_x, line_x], [-1.25,1], transform=ax[1].transAxes, color='gray'))
fig.add_artist(Line2D([line_x, line_x], [-1.25,1], transform=ax[3].transAxes, color='gray'))
fig.add_artist(Line2D([line_x, line_x], [-1.25,1], transform=ax[5].transAxes, color='gray'))


for i, lab in enumerate(caselabels):
    x = ax[2*i+1]
    x.text(-0.05, -0.6, lab, horizontalalignment='right',  rotation=90, transform=x.transAxes)

    
set_monthlabels(ax)

for i, a in enumerate(ax):
    a.text(1.052, 0.935, '('+num_to_lowercase(i)+')', va='center', ha='center', transform=a.transAxes, size=18)

product_name = "SUPP_hourly_three_reactors_with_storage"
    
plt.savefig(gnx.product_path(f"{product_name}.pdf"), bbox_inches='tight', dpi=300)

data = pd.DataFrame(dump).to_numpy()

columns = ", ".join(dump.keys())

header = """Supplemental data: Hourly operation of three reference reactors with medium-cost storage
in the Medium market opportunity scenario
in the PJM_MACC zone.
See cases 324, 10024, and 224.
""" + \
gnx.generic_data_header + \
"""
Values are full-year hourly data series.
Load has been normalized to the peak load (see Step3/Load_data.csv for non-normalized data).
Core power is normalized so that 1 is the peak core power.
Net generation data is normalized so that 1 is the average net power when the core is running at its maximum pace.

""" + \
columns

np.savetxt(gnx.product_path(f"{product_name}.csv"), data, fmt="%.4f", header=header, delimiter=',')

plt.show()


# In[ ]:


cases = [304, 10004, 104] # without storage
#cases = [324, 10024, 224] # with storage

caselabels = ["Pessimistic", "Mid-range", "Optimistic"]
scen = "MedOpp"

top_zones = {case: get_top_zones(gnx.ts_capacity_by_zone(scen, case)) for case in cases}
plant_data = [data_for_reactor_in_zone(scen, case, top_zones[case]) for case in cases]

fig, ax = plt.subplots(2*len(cases)+1, 1, figsize=(16,10), sharex=True)
fig.tight_layout(h_pad=0.2)


dump = dict()

dump["Normalized load"] = plot_electrical_load(fig, ax[0], scen, top_zones[cases[0]])


for i, plant_d in enumerate(plant_data):
    j = 2*i + 1
    dump[key + " core: core power"] = plot_core_power_chart(fig, ax[j], plant_d)
    dump[key + " core: net generation"] =plot_net_generation_chart(fig, ax[j+1], plant_d) 
    
# for i, lab in enumerate(caselabels):    
#     ax[2*i+1].set_ylabel(lab, horizontalalignment='right')

[x.set_ylabel("Core pow.") for x in ax[1::2]]
[x.set_ylabel("Net gen.") for x in ax[2::2]]

line_x = -0.047
fig.add_artist(Line2D([line_x, line_x], [-1.25,1], transform=ax[1].transAxes, color='gray'))
fig.add_artist(Line2D([line_x, line_x], [-1.25,1], transform=ax[3].transAxes, color='gray'))
fig.add_artist(Line2D([line_x, line_x], [-1.25,1], transform=ax[5].transAxes, color='gray'))


for i, lab in enumerate(caselabels):
    x = ax[2*i+1]
    x.text(-0.05, -0.6, lab, horizontalalignment='right',  rotation=90, transform=x.transAxes)

    
set_monthlabels(ax)

for i, a in enumerate(ax):
    a.text(1.052, 0.935, '('+num_to_lowercase(i)+')', va='center', ha='center', transform=a.transAxes, size=18)

product_name = "SUPP_hourly_three_reactors_without_storage"
    
plt.savefig(gnx.product_path(f"{product_name}.pdf"), bbox_inches='tight', dpi=300)

data = pd.DataFrame(dump).to_numpy()

columns = ", ".join(dump.keys())

header = """Supplemental data: Hourly operation of three reference reactors without storage
in the Medium market opportunity scenario
in the PJM_MACC zone.
See cases 304, 10004, and 104.
""" + \
gnx.generic_data_header + \
"""
Values are full-year hourly data series.
Load has been normalized to the peak load (see Step3/Load_data.csv for non-normalized data).
Core power is normalized so that 1 is the peak core power.
Net generation data is normalized so that 1 is the average net power when the core is running at its maximum pace.

""" + \
columns

#np.savetxt(gnx.product_path(f"{product_name}.csv"), data, fmt="%.4f", header=header, delimiter=',')

plt.show()


# # MedOpp, with and without storage

# In[ ]:


cases = [10003, 10023] # without storage

caselabels = ["Without storage", "With storage"]
scen = "MedOpp"

top_zones = {case: get_top_zones(gnx.ts_capacity_by_zone(scen, case)) for case in cases}
plant_data = [data_for_reactor_in_zone(scen, case, top_zones[case]) for case in cases]

fig, ax = plt.subplots(2*len(cases)+3, 1, figsize=(8,6), sharex=True)
fig.tight_layout(h_pad=0.0)


dump = dict()

dump["Normalized load"] = plot_electrical_load(fig, ax[0], scen, top_zones[cases[0]])
ax[0].set_yticks(hours_ticklist)

my_maxprice = 125
dump["No storage: price"] = plot_price(fig, ax[1], scen, -cases[0], top_zones[cases[0]], maxprice=my_maxprice)
dump["No storage: core power"] = plot_core_power_chart(fig, ax[2], plant_data[0])
dump["No storage: net generation"] = plot_net_generation_chart(fig, ax[3], plant_data[0]) 

#dump["Mid-cost storage: price"] = plot_price(fig, ax[4], scen, -cases[1], top_zones[cases[1]], maxprice=my_maxprice)
dump["Mid-cost storage: core power"] = plot_core_power_chart(fig, ax[4], plant_data[1])
dump["Mid-cost storage: storage level"] = plot_storage_chart(fig, ax[5], plant_data[1])
#plot_gross_generation_chart(fig, ax[5], plant_data[1])
dump["Mid-cost storage: net generation"] = plot_net_generation_chart(fig, ax[6], plant_data[1]) 
   

left_label_keys = {'fontsize': left_label_size}
ax[2].set_ylabel("Core pow.", **left_label_keys)
ax[3].set_ylabel("Net gen.", **left_label_keys)

ax[4].set_ylabel("Core pow.", **left_label_keys)
ax[5].set_ylabel("Storage", **left_label_keys)
ax[6].set_ylabel("Net gen.", **left_label_keys)


group_keys = {'rotation': 90, 'horizontalalignment':'right', 'fontsize': (left_label_size+1)}
ax[2].text(-0.09, -0.7, "Without storage", transform=ax[2].transAxes, **group_keys)
ax[-2].text(-0.09, -0.2, "With storage", transform=ax[-2].transAxes, **group_keys)

line_x = -0.08
fig.add_artist(Line2D([line_x, line_x], [-1.25,2.25], transform=ax[2].transAxes, color='gray'))
fig.add_artist(Line2D([line_x, line_x], [-2.5,1.0], transform=ax[-3].transAxes, color='gray'))

[ax[i].my_cbar.set_ylabel('$/MWh     ', labelpad=-10, fontsize=11) for i in [1]]
[ax[i].my_cbar.set_yticks([25,50,75,100], minor=True) for i in [1]]


ax[5].my_cbar.set_ylabel('hours', fontsize=12)

set_monthlabels(ax)


for i, a in enumerate(ax):
    a.text(1.12, 0.935, '('+num_to_lowercase(i)+')', va='center', ha='center', transform=a.transAxes, size=18)

product_name = "Figure_8_hourly"
    
plt.savefig(gnx.product_path(f"{product_name}.pdf"), bbox_inches='tight', dpi=300)

data = pd.DataFrame(dump).to_numpy()

columns = ", ".join(dump.keys())

header = """Figure 8: Hourly operation of the pessimistic reactor with and without storage
in the Medium market opportunity scenario
in the PJM_MACC zone.
See cases 4 and 24
""" + \
gnx.generic_data_header + \
"""
Values are full-year hourly data series.
Load has been normalized to the peak load (see Step3/Load_data.csv for non-normalized data).
Core power is normalized so that 1 is the peak core power.
Storage data is the "fill level" of the thermal storage, measured in hours of the peak core output.
Net generation data is normalized so that 1 is the average net power when the core is running at its maximum pace.
""" + \
columns

np.savetxt(gnx.product_path(f"{product_name}.csv"), data, fmt="%.4f", header=header, delimiter=',')

plt.show()


# # Calculation references in the text

# "The core is operated at _x_ of its maximum annual capacity"

# In[ ]:


a = plant_data[0]['Norm. fusion core power']


# In[ ]:


np.mean(a) / 0.9625


# "Operating above 99% of its capacity x of the time"

# In[ ]:


a_norm = a / np.max(a)
len(a_norm[a_norm > 0.99]) / len(a_norm)


# "The core operates now at xx of its yearly capacity, including running almost constantly starting in July"

# In[ ]:


np.mean(plant_data[1]['Norm. fusion core power']) / 0.9625


# "its ___ hour duration TSS"

# In[ ]:


gnx.ts_duration_for_zone(scen, 10023, 11)


# Allowing the plant to export power at apbout 1.5x of its long-run capacity

# In[ ]:


plt.hist(plant_data[1]['Norm. net fusion generation'], bins=60)
plt.xlim(1,1.5)


# The TSS increases the plant net generation from __ % to __ % of its yearly capacity

# In[ ]:


case_without_tss = 10003
case_with_tss = 10023

hundred_GW = 100 * 1e3 # cases 4, 24, 104, etc are all 100 GW of fusion
fifty_GW = 50 * 1e3 # cases 3, 23, 103, etc are all 100 GW of fusion


# In[ ]:


gw_of_fusion = {4: hundred_GW, 24: hundred_GW, 10003: fifty_GW, 10023: fifty_GW}


# In[ ]:


total_ts_core_cap = np.sum(gnx.ts_capacity_df(scen, case_without_tss).CorePowerCap)
expected_max = gnx.ts_capacity_df(scen, case_without_tss).CorePowerCap[11] / total_ts_core_cap * gw_of_fusion[case_without_tss]
total_net = np.sum(plant_data[0]['Raw net fusion generation']) / 8760

print(f"{(total_net / expected_max)*100:2.2f}")


# In[ ]:


total_ts_core_cap = np.sum(gnx.ts_capacity_df(scen, case_with_tss).CorePowerCap)
expected_max = gnx.ts_capacity_df(scen, case_with_tss).CorePowerCap[11] / total_ts_core_cap * gw_of_fusion[case_with_tss]
total_net = np.sum(plant_data[1]['Raw net fusion generation']) / 8760

print(f"{(total_net / expected_max)*100:2.2f}")


# And increases the value of the fusion core by ___%

# In[ ]:


print(f"{(gnx.grab_data('duals', scen, [10023])[0] - gnx.grab_data('duals', scen, [10003])[0])/gnx.CAPEX_TO_ANN:2.2f}")



# In[ ]:


print(f"{(gnx.grab_data('duals', scen, [case_with_tss])[0] / gnx.grab_data('duals', scen, [case_without_tss])[0] - 1) * 100:2.2f}%")


# In[ ]:




