#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import os

import numpy as np

import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib.lines import Line2D
from matplotlib.patches import Rectangle
from matplotlib.patches import Patch

import genx_fusion_analysis_base as gnx
from genx_fusion_analysis_base import colors

import scipy

import pandas as pd


# Note: these cases have 6 hour ... 1 hour up/down times.

# In[ ]:


nice_gray = "#333333"

mpl.rcParams['font.size'] = 12
mpl.rcParams['axes.titlesize'] = 12


mpl.rcParams['axes.edgecolor'] = nice_gray
mpl.rcParams['axes.titlecolor'] = nice_gray
mpl.rcParams['axes.labelcolor'] = nice_gray
mpl.rcParams['text.color'] = nice_gray
mpl.rcParams['xtick.color'] = nice_gray
mpl.rcParams['ytick.color'] = nice_gray
mpl.rcParams['xtick.labelcolor'] = nice_gray
mpl.rcParams['ytick.labelcolor'] = nice_gray
mpl.rcParams['legend.title_fontsize'] = 11


# In[ ]:


def mean_capacity_factor(scen, resource):
    gen_var = gnx.load_generators_variability(scen)
    return np.mean(gen_var[resource])


# In[ ]:


x_all = np.r_[10,20,50,100,150,200,250,300,350]
x_std = x_all[:-1]


# In[ ]:


a_std = np.arange(1,9)
a_all = np.arange(1, 10)


# In[ ]:


scen_low = "LowOpp"
scen_med = "MedOpp"
scen_high = "HighOpp"
scen_lownonuc = "LowOppNoNucOrCCS"
scen_highnonuc = "HighOppNoNucOrCCS"

cases_nostor_pes = 0 + a_all
cases_nostor_opt = 100 + a_all
cases_nostor_mid = 10000 + a_all


# # Supplementary data: learning curves

# In[ ]:


def max_plant_cost(scen, x, cases):
    duals = gnx.grab_duals(scen, cases)
    gen = gnx.grab_data("generation", scen, cases) / 1e3
    gen_per_cap = gen / x
    return x, (duals + gnx.base_gen_cost * gen_per_cap)/1e3/gnx.CAPEX_TO_ANN


# In[ ]:


def value_curve_interpolation(scen, cases):
    xV, yV = max_plant_cost(scen, x_all, cases)   
    return scipy.interpolate.interp1d(xV, yV, fill_value='extrapolate')


# In[ ]:


def learning_curve_interpolation(scen, cases, progress_ratio, frac_learnable):
    xV, yV = max_plant_cost(scen, x_all, cases)
    caps = np.linspace(x_all[0], x_all[-1])
    c_start = 5e3 # 5.2e3 # yV[0]*0.65
    starting_gw = x_all[0]   
    #yC = frac_learnable* c_start * (caps ** np.log2(progress_ratio) / starting_gw ** np.log2(progress_ratio)) + (1-frac_learnable) *c_start
    yC = frac_learnable * c_start * progress_ratio ** (np.log2(caps/starting_gw)) + (1 - frac_learnable) * c_start
    
    cost_interp = scipy.interpolate.interp1d(caps, yC, fill_value='extrapolate')
    return cost_interp


# In[ ]:


def learning_curve_intersection(scen, cases, progress_ratio=0.9, frac_learnable=0.5, x0=200):   
    val_interp = value_curve_interpolation(scen, cases)
    cost_interp = learning_curve_interpolation(scen, cases, progress_ratio, frac_learnable)
    
    sol = scipy.optimize.fsolve(lambda x: val_interp(x) - cost_interp(x), x0)[0]
    
    if sol > 350:
        sol=350
    
    return sol


# In[ ]:


def learning_curve_area(scen, cases, progress_ratio=0.9, frac_learnable=0.5, x_guess=200):
    val_interp = value_curve_interpolation(scen, cases)
    cost_interp = learning_curve_interpolation(scen, cases, progress_ratio, frac_learnable)
    intersection = learning_curve_intersection(scen, cases, progress_ratio, frac_learnable, x0=x_guess)
    x0 = x_all[0]
    
    area = scipy.integrate.quad(lambda x: val_interp(x) - cost_interp(x), x0, intersection, epsrel=1e-4)
    return area

def learning_curve_annual_value(scen, cases, progress_ratio=0.9, frac_learnable=0.5, x_guess=200):
    r"""
    Billions per year
    """
    return 0.078 * learning_curve_area(scen, cases, progress_ratio, frac_learnable, x_guess)[0] / 1e3


# In[ ]:


def plot_learning_curve_area(ax, scen, cases, progress_ratio=0.9, frac_learnable=0.5, **kwargs):
    val_interp = value_curve_interpolation(scen, cases)
    cost_interp = learning_curve_interpolation(scen, cases, progress_ratio, frac_learnable)
    
    lower = x_all[0]
    x_guess = kwargs.pop('x_guess', 200)
    upper = learning_curve_intersection(scen, cases, progress_ratio, frac_learnable, x_guess)
    x = np.linspace(lower, upper)
    val = learning_curve_annual_value(scen, cases, progress_ratio, frac_learnable, x_guess)
    print(f"{val:3.2f} B / year")
    ax.fill_between(x, val_interp(x), cost_interp(x), **kwargs)


# In[ ]:


def plot_learning_curve(ax, scen, cases, progress_ratio=0.9, frac_learnable=0.5, **kwargs):
    cost_interp = learning_curve_interpolation(scen, cases, progress_ratio, frac_learnable)
    
    lower = x_all[0]
    x_guess = kwargs.pop('x_guess', 200)
    upper = learning_curve_intersection(scen, cases, progress_ratio, frac_learnable, x_guess)
    x = np.linspace(lower, upper)
    return ax.plot(x, cost_interp(x), **kwargs)


# In[ ]:


fig, ax = plt.subplots(1,1, figsize=(11.5,5.5), sharex=True, sharey=True)

styles = ['solid', 'dashed', 'dotted']

generator_style = 'solid'

caps = np.logspace(np.log10(10), np.log10(275))
c_start = 5761
starting_gw = 10

progress_ratio = 0.9
frac_learnable = 0.5

y = frac_learnable* c_start * (caps ** np.log2(progress_ratio) / starting_gw ** np.log2(progress_ratio)) + (1-frac_learnable) *c_start

boringcolor= 'lightgray'

pl_highnonuc, = ax.plot(*max_plant_cost(scen_highnonuc, x_all, cases_nostor_mid), color=nice_gray, ls='-.', label='High market opp, no fission or CCS')

pl_high, = ax.plot(*max_plant_cost(scen_high, x_all, cases_nostor_mid), color=nice_gray, ls=styles[0], label='High market opp.')
pl_med, = ax.plot(*max_plant_cost(scen_med, x_all, cases_nostor_mid), color=nice_gray, ls=styles[1],  label='Med. market opp.')
pl_low, = ax.plot(*max_plant_cost(scen_low, x_all, cases_nostor_mid), color=nice_gray, ls=styles[2],  label='Low market opp.')

profit_color = 'green'
profit_alpha = 0.3

rect = Rectangle((200,7000), 25, 40/gnx.CAPEX_TO_ANN, color=profit_color, alpha=profit_alpha)
ax.add_artist(rect)
ax.text(233, 7055, "$1B / year", color='#444444', va='bottom')

pr = 0.9
fl = 0.5
learn_scen = "MedOpp"
plot_learning_curve_area(ax, learn_scen, cases_nostor_mid, progress_ratio=pr, frac_learnable=fl,
                         color=profit_color, alpha=profit_alpha)
pl_learn, = plot_learning_curve(ax, learn_scen, cases_nostor_mid,
                    progress_ratio=pr,
                    frac_learnable=fl,
                    color='Red',
                    ls='dashed',
                    label="Cost: 10% improvement rate\n on half the initial cost")

legend_x = 1.25


legend_elements = [pl_highnonuc, pl_high,
                   Patch(facecolor=profit_color, alpha=0,
                         label=''),
                   pl_med,
                   
                   Patch(facecolor=profit_color, alpha=profit_alpha,
                         label='Net value'),
                   pl_learn,
                    Patch(facecolor=profit_color, alpha=0,
                         label=''),
                   pl_low,
                  ]

ax.set_xlim(0,350)
maxy = 8500
ax.set_ylim(0, maxy)

cores = ['Optimistic core', 'Mid-range core', 'Pessimistic core'][::-1]
ax2 = ax.twinx()
for ss, sty in enumerate(cores):
    ax2.plot(np.NaN, np.NaN, ls=generator_style,
             label=cores[ss], lw=0.5, c=colors[2-ss])
ax2.get_yaxis().set_visible(False)

legend_kwargs = {'fontsize':12, 'title_fontsize': 12, 'framealpha':1}

ax.set_xlabel("Fusion capacity / GW")
ax.set_ylabel("Capital cost threshold\nfor marginal plant / (USD / kW)")
    
secax = ax.secondary_yaxis('right', functions=(gnx.capex_to_ann, gnx.ann_to_capex), color='gray')
secax.set_ylabel('Annualized cost threshold\nfor marginal plant / (USD / kW·yr)', color='gray')

ax.text(90, 4460, "$17B / year", color='#444444', va='bottom')

leg = ax.legend(handles=legend_elements, **legend_kwargs, loc=(legend_x, 0.32))
leg.set_zorder(100)

plt.tight_layout()
plt.savefig(gnx.product_path('SUPP_fusion_net_value.pdf'), bbox_inches="tight")
plt.show()


# # Output for data storage

# In[ ]:


def round_out(a):
    # return [f"{el:2.1f}" for el in a]
    return a


# In[ ]:


def format_capacity_series(pl):
    return round_out(pl.get_data()[1] * gnx.CAPEX_TO_ANN)


# In[ ]:


data = {"Capacities / GW": round_out(pl_low.get_data()[0]), 
    "LowOpp": format_capacity_series(pl_low),
        "MedOpp": format_capacity_series(pl_med),
        "HighOpp": format_capacity_series(pl_high),
        "HighOppNoNuc": format_capacity_series(pl_highnonuc)}


# In[ ]:


df = pd.DataFrame(data)

data = df.to_numpy()

columns = ", ".join(df.keys())

header = """Supplementary Information Figure 1:
Net value of fusion to the grid.
""" + gnx.generic_data_header + \
f"""
Value of a mid-range reactor without storage in labeled scenarios have units of $ / kW·yr.

The learning curve shown for reactor cost uses the expression
y = 2500 + 2500 * 0.9 ** log2(capacity / (10 GW))
where y is in units of ($ / kW).
To convert from the left axis (capitalized cost) to the units of right axis (annual cost) multiply by {gnx.CAPEX_TO_ANN}.

""" + \
columns
print(header)

np.savetxt(gnx.product_path("SUPP_fusion_net_value.csv"), data, fmt="%.1f", header=header, delimiter=',')


# # Value table

# In[ ]:


scens = ["LowOppLowNuclear", "MedOppLowNuclear", "HighOppLowNuclear",
         "LowOpp", "MedOpp", "HighOpp",
         "LowOppNoNucOrCCS", "HighOppNoNucOrCCS"]


# Note that the below table sometimes has positive values where they should be zero (for the Low Opportunity Scenarios), due to a failed numerical solution.

# In[ ]:


for scen in scens:
    pr = 0.9
    fl = 0.5
    i = learning_curve_intersection(scen, cases_nostor_mid, progress_ratio=pr, frac_learnable=fl)
    a = learning_curve_annual_value(scen, cases_nostor_mid, progress_ratio=pr, frac_learnable=fl, x_guess=200)
    if i < 0:
        i = 0
        
    if a < 0:
        a = 0
        
    pr = 1
    i_none = learning_curve_intersection(scen, cases_nostor_mid, progress_ratio=pr, frac_learnable=fl)
    a_none = learning_curve_annual_value(scen, cases_nostor_mid, progress_ratio=pr, frac_learnable=fl, x_guess=200)
    if i_none < 0:
        i_none = 0
        
    if a_none < 0:
        a_none = 0
    
    print(scen + " & " + str(int(np.round(i_none))) + " & " + str(int(np.round(a_none))) + " & " + str(int(np.round(i))) + " & " + str(int(np.round(a))) + " \\\\ ")


# In[ ]:





# In[ ]:




