#!/usr/bin/env python
# coding: utf-8

# Note that this script can use a large amount of memory,
# so the user might want to comment out some of the commands to produce Supplementary Data plots (at bottom).

# In[ ]:


import genx_fusion_analysis_base as gnx


# In[ ]:


import matplotlib as mpl
import matplotlib.pyplot as plt

import matplotlib.colors as mc

from matplotlib.lines import Line2D
from matplotlib.patches import Patch
import numpy as np
import pandas as pd

import functools


# In[ ]:


prop_cycle = plt.rcParams['axes.prop_cycle']
colors = prop_cycle.by_key()['color']

mpl.rcParams['font.size'] = 14

η=0.4


# In[ ]:


nice_gray = "#333333"

mpl.rcParams['font.size'] = 12
mpl.rcParams['axes.titlesize'] = 12


mpl.rcParams['axes.edgecolor'] = nice_gray
mpl.rcParams['axes.titlecolor'] = nice_gray
mpl.rcParams['axes.labelcolor'] = nice_gray
mpl.rcParams['text.color'] = nice_gray
mpl.rcParams['xtick.color'] = nice_gray
mpl.rcParams['ytick.color'] = nice_gray
mpl.rcParams['xtick.labelcolor'] = nice_gray
mpl.rcParams['ytick.labelcolor'] = nice_gray
plt.rcParams['legend.title_fontsize'] = 11


# In[ ]:


def fusion_net_time_avg_capacity_by_zone(scen, case):
    core_params_df = gnx.load_core_parameters_df(scen)
    
    net_avg_cap_factor = gnx.get_net_avg_cap(core_params_df, case)
    
    peak_thermal_cap = gnx.ts_capacity_by_zone(scen, case)
    return peak_thermal_cap * net_avg_cap_factor


# In[ ]:


def get_category_sums(df, zone_names, variety="Capacity"):
    r"""
    df: Dataframe
        capacity dataframe
    zone_names: list of str
        required to identify generator types
    """
    drop_these = [gnx.FLEX]
    
    if variety == "Capacity" or variety == "Energy":  
        col_to_sum = "EndCap"
    elif variety == "EnergyCapacity":
        col_to_sum = "EndEnergyCap"

    category_sums = dict()

    for idx, row in df.iterrows():
        cat = gnx.generator_category(row.Resource, zone_names)
        val = row[col_to_sum]
        gnx.initialize_if_not_present(category_sums, cat, 0.0)
        category_sums[cat] += val

    gnx.drop_keys(category_sums, drop_these)        
    gnx.remove_nonpositive_entries(category_sums)
    gnx.initialize_if_not_present(category_sums, "Fusion", 0.0)
    gnx.initialize_if_not_present(category_sums, gnx.CCS, 0.0)

    
    return category_sums


# In[ ]:


@functools.cache
def capacity_sum(scen, c):
    zone_names = gnx.get_zone_names()
    df = gnx.get_capacity_df(scen, c)
    
    s = get_category_sums(df, zone_names)
    
    fus_cap = fusion_net_time_avg_capacity_by_zone(scen, c)
    s["Fusion"] = sum(fus_cap)
    
    return s

@functools.cache
def energy_capacity_sum(scen, c):
    zone_names = gnx.get_zone_names()
    df = gnx.get_capacity_df(scen, c)
    
    s = get_category_sums(df, zone_names, variety="EnergyCapacity")
    
    fusion_thermal_storage = gnx.ts_storagecap_by_zone(scen, c)
    s["Fusion"] = sum(fusion_thermal_storage)
    
    return s

def energy_sums(ps_df):
    keys = ps_df.keys()[1:-1]
    esums = ps_df.iloc[1].to_numpy()[1:-1]
    return pd.DataFrame(np.array([keys, esums]).T, columns=["Resource", "EndCap"])
    
def annual_energy_sum(scen, c):
    zone_names = gnx.get_zone_names()
    
    generated_energy_df = energy_sums(gnx.power_annual_sums(scen, c))
    recirculated_energy = gnx.get_total_recirc(scen, c)
    
    generated_energy = get_category_sums(generated_energy_df, zone_names)
    
    net_power = gnx.subtract_dicts(generated_energy, recirculated_energy)
    
    remove_storage_from_energy(net_power)
    return net_power

def remove_storage_from_energy(d):
    stors = ['Hydro-storage', gnx.BATTERY, gnx.METALAIR]
    gnx.drop_keys(d, stors)


# In[ ]:


def dicts_to_dataframe(a):
    r"""
    a: iterable of dicts
    """
    return pd.DataFrame.from_records(a) 


# # Stacked bar plots

# In[ ]:


cap_order = [gnx.BIOMASS, gnx.HYDROELECTRIC, gnx.DISTGEN, gnx.SOLAR, gnx.WIND,  gnx.METALAIR, gnx.BATTERY, gnx.ZCF, gnx.CCS, gnx.NUCLEAR, gnx.FUSION]
energy_order = [gnx.BIOMASS, gnx.HYDROELECTRIC, gnx.DISTGEN, gnx.SOLAR, gnx.WIND,  gnx.ZCF, gnx.CCS, gnx.NUCLEAR, gnx.FUSION]
energycap_order = [gnx.HYDROELECTRIC, gnx.METALAIR, gnx.BATTERY, gnx.FUSION]

def get_sums_for_cases(scen, cases):  
    df = dicts_to_dataframe([capacity_sum(scen, c) for c in cases])
    cap_sums = df[cap_order]
    
    df = dicts_to_dataframe([annual_energy_sum(scen, c) for c in cases])
    energy_sums = df[energy_order]
    
    df = dicts_to_dataframe([energy_capacity_sum(scen, c) for c in cases])
    energycap_sums = df[energycap_order]
    return (cap_sums, energycap_sums, energy_sums,)

scen = "MedOpp"

pes_no_storage = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
mid_no_storage = [0, 10001, 10002, 10003, 10004, 10005, 10006, 10007, 10008, 10009]
opt_no_storage = [0, 101, 102, 103, 104, 105, 106, 107, 108, 109]

cap_sums_nostor, _, _ = get_sums_for_cases(scen, mid_no_storage)

x = cap_sums_nostor.Fusion / 1e3
fusion_capacities = x


# In[ ]:


def nice_stack_plot(ax, x, df, divisor=1e3):
    lists = [df[k].to_numpy()/divisor for k in df.keys()]
    colors =[gnx.color_map[k] for  k in df.keys()]
    ax.stackplot(x, lists, colors=colors, labels=df.keys())
    handles, labels = ax.get_legend_handles_labels()
    return handles[::-1], labels[::-1]    


# In[ ]:


def set_minorxticks(ax):
    ax.set_xticks(fusion_capacities, minor=True)
    ax.set_xlim((0,350))


# In[ ]:


def format_subplot_label(s):
    return '(' + s + ')'


# In[ ]:


labsize=12
x_label = "Fusion capacity / GW"


# In[ ]:


def prepend_keys(df, prepend):
    keys = df.keys()
    pp_keys = [prepend + k for k in list(keys)]
    rn_dict = {k1: k2 for k1, k2 in zip(keys, pp_keys)}
    df.rename(columns=rn_dict, inplace=True)


# In[ ]:


def cap_ecap_en_column(axc, scen, cases):
    cap, ecap, energy = get_sums_for_cases(scen, cases)
    x = cap.Fusion / 1e3
    leghand, leglab = nice_stack_plot(axc[0], x, cap/1e3)
    nice_stack_plot(axc[1], x, ecap/1e3)
    nice_stack_plot(axc[2], x, energy/1e6)
    
    x_label = "Fusion capacity / GW"
    axc[-1].set_xlabel(x_label, size=labsize)
    set_minorxticks(axc[-1])
    axc[0].set_ylim(0, 2.5)
    axc[1].set_ylim(0, 4)
    axc[2].set_ylim(0, 6)
    [ax.spines[side].set_visible(False) for side in ["top", "right"] for ax in axc]

    prepend_keys(cap, "Capacity_")
    prepend_keys(ecap, "StorageCapacity_")
    prepend_keys(energy, "Energy_")
    
    total_column_df = pd.concat([cap, ecap, energy], axis=1)
    
    return leghand, leglab, total_column_df


# In[ ]:


def label_left_side_axes(axs):
    axs[0,0].set_ylabel('Capacity / TW', size=labsize)
    axs[1,0].set_ylabel('Storage / TWh', size=labsize)
    axs[2,0].set_ylabel('Net energy produced / PWh', size=labsize)


# In[ ]:


def no_storage_scenario_plot(scen, caplim=3, storlim=4.5, title=""):
    gridspec_kw={'height_ratios':[1, 0.5, 1]}
    fig, axs = plt.subplots(3, 3, figsize=(10,7),  constrained_layout=True, gridspec_kw=gridspec_kw, sharex=True)

    _, _, col0 = cap_ecap_en_column(axs[:,2], scen, pes_no_storage)
    leghand, leglab, col1 = cap_ecap_en_column(axs[:,1], scen, mid_no_storage)
    leghand, leglab, col2 = cap_ecap_en_column(axs[:,0], scen, opt_no_storage)
    
    prepend_keys(col2, "PessimisticFusion_")
    prepend_keys(col1, "MidrangeFusion_")
    prepend_keys(col0, "OptimisticFusion_")
        

    [ax.set_ylim(0, caplim) for ax in axs[0,:]]

    [ax.set_ylim(0, storlim) for ax in axs[1,:]]

    axs[2,2].legend(leghand, leglab, loc=(1.02,0.0), fontsize=10, frameon=False)

    col_labels = ["Optimistic plant", "Mid-range plant", "Pessimistic fusion plant"]
    [ax.set_title(lab) for lab, ax in zip(col_labels, axs[0,:])]

    label_left_side_axes(axs)
    fig.suptitle(title)
    
    product_name = "SUPP_mix_" + scen + "_no_storage"

    plt.savefig(gnx.product_path(product_name + ".pdf"), bbox_inches='tight')

    plt.show()
    
    df = pd.concat([col0, col1, col2], axis=1)
    data = df.to_numpy()

    columns = ", ".join(df.keys())

    header = f"""Supplemental data:
    
    Mixes of capacities, storage capacities, and annual energy production 
    as a function of the system fusion capacity,
    for each of the three reference plants,
    for the scenario {scen}.
    """ + gnx.generic_data_header + \
    f"""

    Capacity columns have units of MW
    StorageCapacity column have units of MWh
    Energy columns have units of MWh.

    """ + \
    columns

    np.savetxt(gnx.product_path(product_name + ".csv"), data, fmt="%.1f", header=header, delimiter=',')


# In[ ]:


no_storage_scenario_plot("LowOpp", title="Low fusion market opportunity scenario")


# In[ ]:


no_storage_scenario_plot("MedOpp", storlim=4.5, title="Medium fusion market opportunity scenario")


# In[ ]:


no_storage_scenario_plot("HighOpp", title="High fusion market opportunity scenario")


# In[ ]:


no_storage_scenario_plot("HighOppNoNucOrCCS", caplim=3.5, title="High fusion market opportunity scenario; no new fission or gas with CCS")


# In[ ]:


no_storage_scenario_plot("CRLowOpp", caplim=2.5, title="Low fusion market opportunity scenario; Constrained Renewables")


# In[ ]:


no_storage_scenario_plot("CRMedOpp", caplim=2.5, title="Medium fusion market opportunity scenario; Constrained Renewables")


# In[ ]:


no_storage_scenario_plot("CRHighOpp", caplim=2.5, title="High fusion market opportunity scenario; Constrained Renewables")


# In[ ]:


no_storage_scenario_plot("LowOppLowNuclear", title="Low fusion market opportunity scenario; low-cost nuclear")


# In[ ]:


no_storage_scenario_plot("MedOppLowNuclear", title="Medium fusion market opportunity scenario; low-cost nuclear")


# In[ ]:


no_storage_scenario_plot("HighOppLowNuclear", title="High fusion market opportunity scenario; low-cost nuclear")


# In[ ]:




