#!/usr/bin/env python

import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
import pandas as pd
from scipy.stats import linregress

import copy
import functools
import math
import os
import pickle


prop_cycle = plt.rcParams["axes.prop_cycle"]
colors = prop_cycle.by_key()["color"]

cons_corevom = 5
base_η = 0.4
cons_pulse_length = 2  # hours
cons_dwell = 0.15
cons_pass_recirc = 0.2
cons_act_recirc = 0.1
cons_start_energy = 0.05

mid_corevom = 3
mid_pulse_length = 4
mid_dwell = 0.15
mid_pass_recirc = 0.1
mid_act_recirc = 0.1
mid_start_energy = 0.025

base_gen_cost = 39960 + 18750  # USD/MWe/yr
base_gen_vom = 1.74  # USD/MWhe
low_stor_cost = 858
mid_stor_cost = 2 * low_stor_cost
high_stor_cost = 2 * mid_stor_cost

NATURAL_GAS = "Natural Gas"
WIND = "Wind"
NUCLEAR = "Fission"
BATTERY = "Li batteries"
METALAIR = "Metal-air batt."
SOLAR = "Solar"
FUSION = "Fusion"
HYDROELECTRIC = "Hydroelectric"
HYDROPUMPEDSTORAGE = "Hydro. storage"
CCS = "NG-CCS"
ZCF = "ZCF CC & CT"
ZCFCC = "ZCF-CC"
ZCFCT = "ZCF-CT"

DISTGEN = "Distr. Solar"
COAL = "Coal"
FLEX = "Flex. Demand."
BIOMASS = "Biomass"

fusion_marker_generic = "*"
fusion_marker_opt = "P"
fusion_marker_mid = fusion_marker_generic
fusion_marker_pes = 's'
fusion_markers = (fusion_marker_opt, fusion_marker_mid, fusion_marker_pes)
fusion_reactor_names = ("Optimistic", "Mid-range", "Pessimistic")

categories_table = {
    "conventional_steam_coal": COAL,
    "fusion": FUSION,
    "conventional_hydroelectric": HYDROELECTRIC,
    "hydroelectric_pumped_storage": HYDROELECTRIC,
    "nuclear": NUCLEAR,
    "nuclear_nuclear_moderate": NUCLEAR,
    "landbasedwind_class1_moderate": WIND,
    "landbasedwind_class1_advanced": WIND,
    "onshore_wind_turbine": WIND,
    "offshorewind_class1_moderate_fixed": WIND,
    "offshorewind_class1_advanced_fixed": WIND,
    "naturalgas_ccavgcf_moderate": ZCF,
    "naturalgas_ctavgcf_moderate": ZCF,
    "natural_gas_steam_turbine": NATURAL_GAS,
    "natural_gas_fired_combustion_turbine": NATURAL_GAS,
    "natural_gas_fired_combined_cycle": NATURAL_GAS,
    "naturalgas_ccs100_moderate": CCS,
    "other_peaker": "other_peaker",
    "storage_metalair_advanced": METALAIR,
    "storage_metalair_moderate": METALAIR,
    "storage_metalair_conservative": METALAIR,
    "battery_moderate": BATTERY,
    "battery_advanced": BATTERY,
    "utilitypv_class1_moderate": SOLAR,
    "utilitypv_class1_advanced": SOLAR,
    "solar_photovoltaic": SOLAR,
    "ldv_mw": FLEX,
    "res_wh_mw": FLEX,
    "distributed_generation": DISTGEN,
    "biomass": BIOMASS,
}

color_map = {
    FUSION: "Pink",
    SOLAR: "Khaki",
    WIND: "Lightblue",
    ZCF: "darkseagreen",
    ZCFCC: "darkseagreen",
    ZCFCT: "darkseagreen",
    CCS: "Silver",
    NUCLEAR: "Purple",
    BATTERY: "peru",
    METALAIR: "firebrick",
    NATURAL_GAS: "Gray",
    DISTGEN: "wheat",
    BIOMASS: "Green",
    HYDROELECTRIC: "Teal",
    HYDROPUMPEDSTORAGE: "Teal",
    "Other": "#F0F0F0",
    FLEX: "lawngreen",
}

ZONE_NAMES = [
    "FRCC",
    "ISONE",
    "MISO_ILINKY",
    "MISO_LRZ123",
    "MISO_LRZ7",
    "MISO_LRZ8910",
    "MIS_MO",
    "NY_E",
    "NY_W",
    "PJM_COMD",
    "PJM_Dom",
    "PJM_MACC",
    "PJM_WEST",
    "SPP_N",
    "SPP_NN",
    "SPP_S",
    "S_C_TVA",
    "S_D_AECI",
    "S_SOU",
    "S_VACA",
]

FUSION_RESOURCES = [z.lower() + "_fusion" for z in ZONE_NAMES]


def make_data_dict(orig_data):
    r"""Repackaging a dict of dicts outside-in

    orig_data: original data
    """
    data = dict()
    duals = orig_data["Duals"]
    for k, v in duals.items():
        data[k] = dict()
        data[k]["fusion_peak_cap"] = orig_data["Peak core cap"][k]
        data[k]["duals"] = duals[k]
        data[k]["generation"] = orig_data["Generation"][k]
        data[k]["generation_ratios"] = orig_data["Generation ratio"][k]
        data[k]["storage_capacity"] = orig_data["Storage capacity"][k]
        data[k]["duration"] = orig_data["Duration"][k]
        data[k]["total_costs"] = orig_data["Total cost"][k]
        data[k]["avoided_costs"] = (
            orig_data["Total cost"][0] - orig_data["Total cost"][k]
        )
    return data


BASE_SCENARIOS_FOLDER = "../Step2_base_scenarios"
SCENARIO_TEMPLATE_FOLDER = "../Step3_case_inputs"
RESULTS_FOLDER = "../Step4_selected_scenario_outputs"
PRODUCTS_FOLDER = "../Step6_products"


def load_scenario_data(folder):
    path = os.path.join(RESULTS_FOLDER, folder, "scenario_data.pkl")

    with open(path, "rb") as handle:
        raw_data = pickle.load(handle)
        data = make_data_dict(raw_data)

    return data


def product_path(name):
    return os.path.join(PRODUCTS_FOLDER, name)


scenarios_to_load = [
    "LowOppLowNuclear",
    "MedOppLowNuclear",
    "HighOppLowNuclear",
    "LowOpp",
    "MedOpp",
    "HighOpp",
    "LowOppNoNuc",
    "MedOppNoNuc",
    "HighOppNoNuc",
    "LowOppNoNucOrCCS",
    "HighOppNoNucOrCCS",
    "CRLowOpp",
    "CRMedOpp",
    "CRHighOpp",
    "MedOppMustRun",
]

all_scenario_data = {k: load_scenario_data(k) for k in scenarios_to_load}

# database access


def grab_data(key, scenario, indices):
    data = all_scenario_data
    return np.array([data[scenario][i][key] for i in indices])


# specialized database access

CAPEX_TO_ANN = 0.078
MEGA_TO_KILO = 1e3

capex_scale = MEGA_TO_KILO * CAPEX_TO_ANN

grab_duals = functools.partial(grab_data, "duals")
grab_duals.__doc__ = r"""Dual values of the case, in $/MW⋅yr"""

grab_generation = functools.partial(grab_data, "generation")
grab_generation.__doc__ = r"""Gross generation for the case, in MW"""

grab_duration = functools.partial(grab_data, "duration")
grab_duration.__doc__ = r"""Storage hours for the case in core-hours"""


def core_value_capex(scenario, cases):
    r"""CAPEX in $/kW for the core alone"""
    return grab_data("duals", scenario, cases) / capex_scale


def grab_generation_ratios(scenario, indices):
    return grab_data("generation_ratios", scenario, indices) / base_η


def grab_averagevalue(scenario, indices, capacities):
    return grab_data("avoided_costs", scenario, indices) / capacities


# tokamak calculations


def active_fraction(pulse_length, dwell_time):
    """Fraction of the time that the reactor can be on

    Parameters
    ----------
    pulse_length : array_like
    dwell_time : array_like

    The two parameters should have the same dimensions.
    They can each be floats.

    Returns
    -------
    array_like
    """
    if isinstance(pulse_length, np.ndarray):
        return np.array(
            [active_fraction(*z) for z in zip(pulse_length, dwell_time)]
        )

    if pulse_length > 0:
        active_frac = 1 - dwell_time / pulse_length
    else:
        active_frac = 1
    return active_frac


def average_startup_power(pulse_length, start_energy):
    """Time-averaged startup power

    Parameters
    ----------
    pulse_length : array_like
    start_energy : array_like

    The two parameters should have the same dimensions.
    They can each be floats.

    Returns
    -------
    array_like
    """
    if isinstance(pulse_length, np.ndarray):
        lz = [
            average_startup_power(*z) for z in zip(pulse_length, start_energy)
        ]
        return np.array(lz)

    if pulse_length > 0:
        avg_start_power = start_energy / pulse_length
    else:
        avg_start_power = 0
    return avg_start_power


def capacityfactor(
    pulse_length, dwell_time, passive_recirc, active_recirc, start_energy
):
    """Time-averaged net capacity factor

    The ratio of the peak gross instanteneous capacity to the time-averaged net
    capacity

    Parameters
    ----------
    pulse_length : array_like
    dwell_time : array_like
    passive_recirc : array_like
    active_recirc : array_like
    start_energy : array_like

    The arguments should have the same dimensions.
    They can each be floats.

    Returns
    -------
    array_like
    """
    active_frac = active_fraction(pulse_length, dwell_time)
    avg_start_power = average_startup_power(pulse_length, start_energy)
    net_th_frac = (
        active_frac * (1 - active_recirc) - passive_recirc - avg_start_power
    )
    return net_th_frac


def effectiveVOMcost(
    vomMWhth,
    eff,
    pulse_length,
    dwell_time,
    passive_recirc,
    active_recirc,
    start_energy,
):
    """Total effective VOM cost for the core only

    The cost of a marginal quantity of electricity.
    Assumes that the core is running at its maximum cadence.

    Parameters
    ----------
    vomMWhth : array_like
        Core VOM cost in $/MWh(th)
    dwell_time : array_like
        Fractions of an hour
    passive_recirc : array_like
        Fraction of peak instanteneous gross capacity
    active_recirc : array_like
        Fraction of peak instanteneous gross capacity
    start_energy : array_like
        Fraction of peak instanteneous gross capacity multiplied by 1 hour

    The arguments should have the same dimensions.
    They can each be floats.

    Returns
    -------
    array_like

    See also
    --------
    totalVOMcost : total plant VOM cost
    """
    active_frac = active_fraction(pulse_length, dwell_time)
    cf = capacityfactor(
        pulse_length, dwell_time, passive_recirc, active_recirc, start_energy
    )
    vomMWh_nete = active_frac * vomMWhth / (cf * eff)
    return vomMWh_nete


def gross_to_net_generation(
    pulse_length, dwell_time, passive_recirc, active_recirc, start_energy
):
    active_frac = active_fraction(pulse_length, dwell_time)
    cf = capacityfactor(
        pulse_length, dwell_time, passive_recirc, active_recirc, start_energy
    )
    return active_frac / cf


def gen_vom(
    pulse_length, dwell_time, passive_recirc, active_recirc, start_energy
):
    gross_to_net = gross_to_net_generation(
        pulse_length, dwell_time, passive_recirc, active_recirc, start_energy
    )
    return base_gen_vom * gross_to_net


def totalVOMcost(
    vomMWhth,
    eff,
    pulse_length,
    dwell_time,
    passive_recirc,
    active_recirc,
    start_energy,
):
    """Total effective VOM cost for the plant

    The cost of a marginal quantity of electricity.
    Assumes that the plant is running at its maximum cadence.

    Parameters
    ----------
    vomMWhth : array_like
        Core VOM cost in $/MWh(th)
    eff : array_like
        Thermal efficiency
    pulse_length : array_like
        Hours. Must be an integer.
    dwell_time : array_like
        Fractions of an hour
    passive_recirc : array_like
        Fraction of peak instanteneous gross capacity
    active_recirc : array_like
        Fraction of peak instanteneous gross capacity
    start_energy : array_like
        Fraction of peak instanteneous gross capacity multiplied by 1 hour

    The arguments should have the same dimensions.
    They can each be floats.

    Returns
    -------
    array_like, $/MWh

    See also
    --------
    effectiveVOMcost : VOM cost of the core
    """
    core_vom_cost = effectiveVOMcost(
        vomMWhth,
        eff,
        pulse_length,
        dwell_time,
        passive_recirc,
        active_recirc,
        start_energy,
    )
    gen_vom_cost = gen_vom(
        pulse_length, dwell_time, passive_recirc, active_recirc, start_energy
    )
    return core_vom_cost + gen_vom_cost


@functools.cache
def load_core_parameters_df(scen):
    r"""
    Only works for pulsed tokamaks, returns nonsense of CoreMaxUp = -1
    """
    η = base_η
    path = os.path.join(inputs_folder_path(scen), "replacements.csv")
    df = pd.read_csv(path)

    df["CoreActiveFraction"] = 1 - df.CoreDwellTime / df.CoreMaxUp
    df["CoreNetAvgCap"] = (
        df.CoreActiveFraction * (1 - df.CoreRecircAct)
        - df.CoreRecircPass
        - df.CoreStartEnergy / df.CoreMaxUp
    )

    df["Thermal-peak-to-net-avg-cap"] = 1 / (η * df.CoreNetAvgCap)
    df["CoreThermalNetAvgCap"] = η * df.CoreNetAvgCap

    df["f_recirc"] = (
        df.CoreActiveFraction * df.CoreRecircAct
        + df.CoreRecircPass
        + df.CoreStartEnergy / df.CoreMaxUp
    ) / df.CoreActiveFraction
    df["GrossToNetGeneration"] = 1 / (1 - df["f_recirc"])

    df["LongRunGenToCoreRatio"] = η * df["CoreActiveFraction"]

    key = "MinimumNetPowerWithStorageRelativeToPeakGrossGeneration"
    df[key] = (
        -df.CoreRecircPass
        - df.CoreRecircAct
        + np.minimum(
            0, -df.CoreStartEnergy + df.CoreRecircAct * df.CoreDwellTime
        )
    )

    return df


def lookup_by_column(df, kase, col):
    return df[df.Case == kase][col].to_numpy()[0]

def get_active_fraction(df, kase):
    key = "CoreActiveFraction"
    return lookup_by_column(df, kase, key)

def get_net_avg_cap(df, kase):
    k = "CoreThermalNetAvgCap"
    return lookup_by_column(df, kase, k)


def get_thermal_peak_to_net_avg_cap(df, kase):
    k = "Thermal-peak-to-net-avg-cap"
    return lookup_by_column(df, kase, k)


def get_gross_to_net_generation(df, kase):
    k = "GrossToNetGeneration"
    return lookup_by_column(df, kase, k)


def get_long_run_gen_to_core_ratio(df, kase):
    k = "LongRunGenToCoreRatio"
    return lookup_by_column(df, kase, k)


def get_minimum_net_power_fraction(df, kase):
    k = "MinimumNetPowerWithStorageRelativeToPeakGrossGeneration"
    return lookup_by_column(df, kase, k)


def get_passive_recirc_fraction(df, kase):
    r"""For a plant design, from the Design Parameters df & Case No,
    """
    k = "CoreRecircPass"
    return lookup_by_column(df, kase, k)


def storage_cost(df: pd.DataFrame, kase: int) -> float:
    """Lookup from Design Parameters df"""
    k = "StorFixedCostperMWhth"
    return lookup_by_column(df, kase, k)


def core_vom_cost(df: pd.DataFrame, kase: int) -> float:
    """Lookup from Design Parameters df"""
    k = "CoreVarOMCostperMWhth"
    return lookup_by_column(df, kase, k)


# Loading map data


def load_centroids():
    df = pd.read_csv("EI_centroids_modified.csv")
    df.drop("centroid", axis=1, inplace=True)
    return df


def centroid_for_zone(z, df=None):
    r"""Return (lon, lat)"""
    if df is None:
        df = load_centroids()

    return (df["lon"][z - 1], df["lat"][z - 1])


def list_of_centroids(centroids_df):
    lats = centroids_df.lat.to_numpy()
    lons = centroids_df.lon.to_numpy()
    return np.array([lons, lats]).T


# Loading from the template

def inputs_folder_path(scen):
    return os.path.join(SCENARIO_TEMPLATE_FOLDER, scen)


@functools.cache
def load_from_template(filename, scen):
    path = os.path.join(inputs_folder_path(scen), "template", filename)
    df = pd.read_csv(path)
    return df


def parentscen(scen):
    start = "2035_"
    for opp in ["Low", "Med", "High"]:
        if opp in scen:
            return start + opp


def base2020_folder_path(scen):
    """Loads from the 2020–2035 simulations"""
    scen_par = parentscen(scen)
    return os.path.join(BASE_SCENARIOS_FOLDER, scen_par)


@functools.cache
def load_from_base2020(filename, scen):
    path = os.path.join(base2020_folder_path(scen), filename)
    df = pd.read_csv(path)
    return df


load_load_data_df = functools.partial(load_from_template, "Load_data.csv")
load_generators_variability = functools.partial(
    load_from_template, "Generators_variability.csv"
)
load_fuels_data = functools.partial(load_from_template, "Fuels_data.csv")
load_generators_data_raw = functools.partial(
    load_from_template, "Generators_data.csv"
)
load_original_network_connections = functools.partial(
    load_from_template, "Network.csv"
)
load_base2020_network_connections = functools.partial(
    load_from_base2020, "Network.csv"
)


@functools.cache
def load_data_for_zone(zz, scen):
    r"""Zones start at 1 ..."""
    df = load_load_data_df(scen)
    key = "Load_MW_z" + str(zz)
    return df[key].to_numpy()


def num_transmission_lines(network):
    """
    Parameters
    ----------
    network : pd.DataFrame

    Returns
    -------
    int
    """
    return len(network["Network_Lines"])


def zz_zone_to_integer(zz_zone: str):
    """ 'z19' to 19
    Returns
    -------
    one-indexed zone
    """
    return int(zz_zone[1:])


def num_zones_from_network(network):
    """
    Parameters
    ----------
    network : pd.DataFrame

    Returns
    -------
    int
    """
    zones_list = network.Network_zones
    return len(zones_list[zones_list.notna()])


def line_endpoints(network, line):
    """Zones that a line joins
    Parameters
    ----------
    network : pd.DataFrame
    line : int
        zero-indexed

    Returns
    -------
    tuple of zero-indexed zones
    """
    num_zones = num_zones_from_network(network)

    zz_zones = ["z"+str(i) for i in range(1, num_zones+1)]

    endpoints = network[zz_zones].iloc[line]
    start = endpoints[endpoints == 1]
    end = endpoints[endpoints == -1]
    start_zone_ix = zz_zone_to_integer(start.keys()[0])
    end_zone_ix = zz_zone_to_integer(end.keys()[0])

    return (start_zone_ix-1, end_zone_ix-1)


def mean_capacity_factor_from_variability(gen_var, g):
    r"""Mean capacity factor

    Parameters
    ----------
    gen_var : pd.DataFrame
        Generators variability
    g : string
        resource
    """
    return np.mean(gen_var[g])


def augment_generators_data(gen_dat, gen_var):
    r"""Add a mean capacity factor column to generators_data

    Parameters
    ----------
    gen_dat: pd.DataFrame
    gen_var: pd.DataFrame

    Returns
    -------
    Nothing. Modifies the gen_dat dataframe.
    """
    newcol = "MeanCapFactor"
    G = len(gen_dat)
    gen_dat[newcol] = np.ones(G)
    for i, g in enumerate(gen_dat.Resource):
        mcf = mean_capacity_factor_from_variability(gen_var, g)
        gen_dat.loc[i, newcol] = mcf


@functools.cache
def load_generators_data(scen):
    r"""Fancy load + augementation, with caching"""
    gendat = load_generators_data_raw(scen)
    genvar = load_generators_variability(scen)
    augment_generators_data(gendat, genvar)

    return gendat


def all_resources(scen):
    """List all the resources"""
    df = load_generators_data_raw(scen)
    return list(df.Resource)


# Loading from the esults


@functools.cache
def load_case_result(filename: str, scen: str, kase: int, parent=False):
    r"""Loads a particular file in a case & scenario into a dataframe.

    Parameters
    ----------
    filename: string
        typically xyz.csv
    kase: int
        case number
    scen: string
        relative path
    parent: bool
        If True, load 2020-2035 results instead of 2035-2050 results

    Returns
    -------
    pd.DataFrame
    """
    path = os.path.join(RESULTS_FOLDER, scen, "case_" + str(kase), filename)

    if parent:
        scen_par = parentscen(scen)
        path = os.path.join(BASE_SCENARIOS_FOLDER, scen_par, "Results", filename)

    df = pd.read_csv(path)
    return df


ts_capacity_df = functools.partial(load_case_result, "TS_capacity.csv")


def ts_capacity_by_zone(scen, kase):
    return ts_capacity_df(scen, kase).CorePowerCap.to_numpy()


def ts_storagecap_by_zone(scen, kase):
    return ts_capacity_df(scen, kase).TSEnergyCap.to_numpy()


def ts_duration_for_zone(scen, kase, zone):
    a = ts_capacity_by_zone(scen, kase)
    b = ts_storagecap_by_zone(scen, kase)
    return (b/a)[zone]


def fusion_corepower(scen, kase):
    return load_case_result("TSCorePwr.csv", scen, kase)


def fusion_corepower_hourly(scen, kase):
    df = fusion_corepower(scen, kase)
    return df[2:]


def fusion_corepower_totals_by_zone(scen, kase):
    df = fusion_corepower(scen, kase)
    return df.iloc[1][FUSION_RESOURCES]


def ts_soc_df(scen, kase):
    return load_case_result("TS_SOC.csv", scen, kase)[2:]


def get_capacity_df(scen, kase):
    df = load_case_result("capacity.csv", scen, kase)
    G = len(df) - 1
    return df.head(G)


power_dataframe = functools.partial(load_case_result, "power.csv")
charge_dataframe = functools.partial(load_case_result, "charge.csv")


def power_annual_sums(scen, kase):
    """Gross power generated annually"""
    df = power_dataframe(scen, kase)
    return df.head(2)


def power_hourly(scen, kase):
    """Gross power hour-by-hour"""
    df = power_dataframe(scen, kase)
    return df[2:]


def charge_hourly(scen, kase):
    """Charging hour-by-hour"""
    df = charge_dataframe(scen, kase)
    return df[2:]


def power_hourly_fusion(scen, kase):
    """Gross power hour-by-hour"""
    df = power_dataframe(scen, kase)
    return df[FUSION_RESOURCES][2:]


recirculating_power_df = functools.partial(load_case_result, "TS_Recirc.csv")


def get_total_recirc(scen, kase):
    """Annual recirculating power"""
    df = recirculating_power_df(scen, kase)
    return {"Fusion": df.iloc[1].to_numpy()[-1]}


def fusion_recirc(scen, kase, in_zone=None):
    """Hourly recirculating power"""
    df = recirculating_power_df(scen, kase)
    hdf = df[FUSION_RESOURCES][2:]
    if isinstance(in_zone, int):
        return hdf.iloc[:, in_zone].to_numpy()
    else:
        return hdf


def get_zonal_recirc(scen, kase):
    df = recirculating_power_df(scen, kase)
    return df.iloc[1][1:-1].to_numpy()


load_price_data_df = functools.partial(load_case_result, "prices.csv")
load_gen_starts_df = functools.partial(load_case_result, "start.csv")
load_fusion_starts_df = functools.partial(load_case_result, "f_start.csv")
load_costs_df = functools.partial(load_case_result, "costs.csv")
load_capacityvalue_df = functools.partial(
    load_case_result, "CapacityValue.csv")
load_reservemargin_df = functools.partial(
    load_case_result, "ReserveMargin.csv")
load_reservemarginrevenue_df = functools.partial(
    load_case_result, "ReserveMarginRevenue.csv")

load_tlosses_df = functools.partial(load_case_result, "tlosses.csv")


@functools.cache
def tlosses_hourly(scen, kase, line=None):
    df = load_tlosses_df(scen, kase)
    hdf = df[1:]
    if isinstance(line, int):
        lline_str = str(line + 1)
        return hdf[lline_str]
    else:
        return hdf


def total_system_cost(scen, kase):
    r"""The model objective
    """
    df = load_costs_df(scen, kase)
    return df["Total"][0]


def total_cost_savings_vs_reference(scen, kase, reference_case=0):
    reference_cost = total_system_cost(scen, reference_case)
    my_cost = total_system_cost(scen, kase)
    return reference_cost - my_cost


def fusion_total_reserve_margin_revenue_by_zone(scen: str, kase: int):
    """
    Returns
    -------
    pandas.Series
    """
    df = load_reservemarginrevenue_df(scen, kase)
    num_zones = len(ZONE_NAMES)
    return df["AnnualSum"][-num_zones:]


def fusion_total_reserve_margin_revenue(scen, kase):
    """
    Returns
    -------
    float
    """
    return sum(fusion_total_reserve_margin_revenue_by_zone(scen, kase))


def fusion_capacity_value(scen, kase):
    r"""
    Returns
    -------
    20 x 8760 numpy array
    """
    df = load_capacityvalue_df(scen, kase)
    hour_columns = ["t"+str(i) for i in range(1, 8761)]
    hourly_data = df[hour_columns]
    num_crm_zones = 6
    G = len(df) // num_crm_zones
    restacked_data = np.sum(
        [hourly_data.iloc[i*G:(i+1)*G].to_numpy() for i in
            range(num_crm_zones)], axis=0)
    restacked_data_fusion = restacked_data[-20:]
    return restacked_data_fusion


def fusion_capacity_value_total(scen, kase):
    return np.sum(fusion_capacity_value(scen, kase))


def load_network_expansion(scen, kase, parent=False):
    df = load_case_result("network_expansion.csv", scen, kase, parent)
    df = copy.deepcopy(df)

    # This corrects capacities from GW to MW, to match Network.csv.
    # And corrects yearly costs from $M to $.
    GENX_ISSUE_222_IS_NOT_FIXED = True
    if GENX_ISSUE_222_IS_NOT_FIXED:
        df.New_Trans_Capacity *= 1e3
        df.Cost_Trans_Capacity *= 1e6

    return df


def energy_revenue_df(scen, kase):
    return load_case_result("EnergyRevenue.csv", scen, kase)


# --------------------------------------------


def capex_to_ann(x):
    return x * CAPEX_TO_ANN


def ann_to_capex(x):
    return x / CAPEX_TO_ANN


# ------------------- working with zones


def get_zone_names():
    """Zone names sorted by zone number.

    Returns
    -------
    list
    """
    return ZONE_NAMES


def detect_substring(main_string, substrings):
    r"""First of the substrings found in the main string

    Note: right now doesn't assess the position of the substring,
    i.e. whether it's a prefix.
    """
    for s in substrings:
        if s in main_string or s.lower() in main_string:
            return s


def detect_zone_name(generator, zone_names):
    r"""Returns the first zone name string found in the generator
    FRCC_conventional_hydroelectric_1 --> FRCC
    """
    return detect_substring(generator, zone_names)


def remove_zone_name(generator, zone_names):
    r"""Strips a resource name of its zonal prefix

    FRCC_wind_1 --> wind_1

    Notes
    -----
    Need to go backward in order to catch SPP_NN before SPP_N
    """
    zone_name = detect_zone_name(generator, zone_names[::-1])
    zone_prefix = zone_name + "_"
    return generator[len(zone_prefix):]


def remove_number_suffixes(a_string):
    r"""
    offshorewind_class1_moderate_fixed_0_180_1 ->
        offshorewind_class1_moderate_fixed
    """
    segs = a_string.split("_")
    for i, s in enumerate(segs[::-1]):
        if i == 0 and not s.isnumeric():
            return "_".join(segs)
        if i > 0 and not s.isnumeric():
            return "_".join(segs[:-i])


def generator_base_name(generator, zone_names):
    r"""
    S_VACA_offshorewind_class1_moderate_fixed_0_180_1 ->
        offshorewind_class1_moderate_fixed
    """
    return remove_number_suffixes(remove_zone_name(generator, zone_names))


def generator_category(g, zone_names):
    r"""Assign a generator to one of the categories in categories_table,
    i.e. wind, solar, battery, nuclear.
    """
    bn = generator_base_name(g, zone_names)
    return categories_table[bn]


def initialize_if_not_present(d, k, iv):
    r"""
    Parameters
    ----------
    d : dict
    k : key
    iv: initialization value
    """
    if k not in d.keys():
        d[k] = iv


def remove_nonpositive_entries(d):
    """
    Parameters
    ----------
    d : dict
    """
    to_delete = [k for k, v in d.items() if v < 0 or math.isclose(v, 0)]
    for k in to_delete:
        del d[k]


def drop_keys(d, keys):
    r"""
    Parameters
    ----------
    d : dict
    keys : list
    """
    for k in keys:
        if k in d:
            del d[k]


# ------------------- working with power and energy sums


def subtract_dicts(d1, d2):
    r"""d2 keys are a subset of d1 keys

    Parameters
    ----------
    d1 : dict
    d2 : dict
    """
    diffs = dict()
    for k, v in d1.items():
        if k in d2:
            diffs[k] = v - d2[k]
        else:
            diffs[k] = v
    return diffs

# ------------------------------


def anncost_from_lcoe(lcoe, var, capacityfactor=1):
    return (lcoe - var) * 8760 * capacityfactor / 1000


def plot_lcoe_line(ax, lcoe=60, capacityfactor=1, color='k', lw=0.5, **kwargs):
    r"""
    lcoe in $/MWh

    x axis in $/(Mw⋅h)
    y axis in $/(kw⋅yr)
    """
    varom = np.linspace(0, lcoe)
    ax.plot(varom, anncost_from_lcoe(lcoe, varom, capacityfactor=capacityfactor),
            color=color, lw=lw, **kwargs)

# ------------------------------
# Data headers


generic_data_header = """
Data for "The value of fusion energy to a decarbonized United States electric grid"
J. A. Schwartz, W. Ricks, E. Kolemen, J. Jenkins
Submitted to Joule, 2022
"""
