#!/usr/bin/env python
# coding: utf-8

# # Use this to aggregate the IPM regions into your model's regions

# We take the original shapefile and do a union on the regions, then save as a new shapefile.

# In[ ]:


import shapely

import geopandas as gpd
import pandas as pd

from geopandas.geodataframe import GeoDataFrame


# **You'll need to have these files**

# In[ ]:


original_ipm_regions_file = "/home/jacob/Desktop/ipm_regions/IPM_Regions_201770405.shp"

# This is just used to a list of the model's region names.
# See the first printed output below.
# This could be done many other ways as well.
model_centroid_file = "EI_centroids_modified.csv"


# In[ ]:


# get a list of the model regions as strings

def get_list_of_model_regions(model_centroid_file):
    """
    This is just used to a list of the model's region names.
    This could be done many other ways as well.
    """
    def load_centroids():
        df = pd.read_csv(model_centroid_file)
        df.drop("centroid", axis=1, inplace=True)
        return df
    
    centroids_df = load_centroids()

    model_regions = centroids_df.model_region.to_numpy()
    return model_regions

model_regions = get_list_of_model_regions(model_centroids_file)
print(model_regions)


# In[ ]:


region_aggregations = {"ISONE": ["NENG_ME", "NENG_CT", "NENGREST"],
                      "NY_W": ["NY_Z_A", "NY_Z_B", "NY_Z_C&E", "NY_Z_D"],
                      "NY_E": ["NY_Z_F", "NY_Z_G-I", "NY_Z_J", "NY_Z_K"],
                      "PJM_MACC": ["PJM_WMAC", "PJM_EMAC", "PJM_SMAC", "PJM_PENE"],
                      "PJM_WEST": ["PJM_West", "S_C_KY", "PJM_AP", "PJM_ATSI"],
                      "MISO_LRZ123": ["MIS_MAPP", "MIS_MNWI", "MIS_WUMS", "MIS_MIDA", "MIS_IA"],
                      "MISO_ILINKY": ["MIS_IL", "MIS_INKY"],
                      "MISO_LRZ7": ["MIS_LMI"],
                      "MISO_LRZ8910": ["MIS_AR", "MIS_LA", "MIS_AMSO", "MIS_WOTA", "MIS_D_MS"],
                      "SPP_NN": ["SPP_WAUE", "SPP_NEBR"],
                      "SPP_S": ["SPP_SPS", "SPP_WEST"]}


# In[ ]:


# This may take some time as the ipm regions file is large.
ipm = gpd.read_file(original_ipm_regions_file)


# In[ ]:


def lookup_index(gdf, region):
    return gdf.index[gdf.IPM_Region == region].to_list()[0]

def joined_regions(gdf, indices):
    r"""
    gdf: GeoDataFrame
        The original model regions
    indices: list of ints
        Indices of regions to join together
    """    
    s = gdf.iloc[indices]
    return s.unary_union


# In[ ]:


region_aggregations_indices = dict()
for k, v in region_aggregations.items():
    region_aggregations_indices[k] = [lookup_index(ipm, r) for r in v]
    


# In[ ]:


def make_geodataframe(gdf, region, region_aggregations_indices):
    r"""
    
    gdf: GeoDataFrame
        The original model regions
    region: string
        Name of one particular region in the final model
    region_aggregation_indices: dict
        Example:
           {aggregated_region_1: [1, 3, 4],
            aggregated_region_2: [2, 5]}
            
    Returns
    -------
    a single-row new GeoDataFrame representing one region of the final model
    """
    reg_col = "IPM_Region"
    geom_col = "geometry"
       
    if region in region_aggregations_indices.keys():
        new_region = joined_regions(gdf, region_aggregations_indices[region])
    else:
        reg_ix = lookup_index(gdf, region)
        new_region = gdf.geometry[reg_ix]
        
    my_dict = {reg_col: [region], geom_col:[new_region]}
        
    return GeoDataFrame(my_dict)


# In[ ]:


agg_gdf = pd.concat([make_geodataframe(ipm,
                                       model_region,
                                       region_aggregations_indices) for model_region in model_regions])


# In[ ]:


agg_gdf.to_file("My_Model_regions.shp")


# Check that we can read the new file.

# In[ ]:


# ipm_new = gpd.read_file("My_Model_regions.shp")

